@@ -3,7 +3,7 @@
  * File name: MainWindow.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -23,12 +23,13 @@
  */
 
 #include "MainWindow.h"
+#include "GeonkickModel.h"
 #include "oscillator.h"
 #include "envelope_widget.h"
 #include "oscillator_group_box.h"
 #include "general_group_box.h"
 #include "control_area.h"
-#include "top_bar.h"
+#include "TopBar.h"
 #include "limiter.h"
 #include "export_widget.h"
 #include "geonkick_api.h"
@@ -39,17 +40,22 @@
 #include <RkEvent.h>
 
 constexpr int MAIN_WINDOW_WIDTH  = 940;
+#ifdef GEONKICK_SINGLE
+constexpr int MAIN_WINDOW_HEIGHT = 680;
+#else
 constexpr int MAIN_WINDOW_HEIGHT = 705;
+#endif // GEONKICK_SINGLE
 
-MainWindow::MainWindow(RkMain *app, GeonkickApi *api, const std::string &preset)
+MainWindow::MainWindow(RkMain& app, GeonkickApi *api, const std::string &preset)
         : GeonkickWidget(app)
         , geonkickApi{api}
         , topBar{nullptr}
         , envelopeWidget{nullptr}
         , presetName{preset}
         , limiterWidget{nullptr}
-        , kitModel{new KitModel(this, geonkickApi)}
+        , geonkickModel{new GeonkickModel(this, geonkickApi)}
 {
+        setTitle(Geonkick::appName);
         setName("MainWindow");
         setScaleFactor(geonkickApi->getScaleFactor());
         createViewState();
@@ -60,14 +66,14 @@ MainWindow::MainWindow(RkMain *app, GeonkickApi *api, const std::string &preset)
         createShortcuts();
 }
 
-MainWindow::MainWindow(RkMain *app, GeonkickApi *api, const RkNativeWindowInfo &info)
+MainWindow::MainWindow(RkMain& app, GeonkickApi *api, const RkNativeWindowInfo &info)
         : GeonkickWidget(app, info)
         , geonkickApi{api}
         , topBar{nullptr}
         , envelopeWidget{nullptr}
         , presetName{std::string()}
         , limiterWidget{nullptr}
-        , kitModel{new KitModel(this, geonkickApi)}
+        , geonkickModel{new GeonkickModel(this, geonkickApi)}
 {
         setScaleFactor(geonkickApi->getScaleFactor());
         createViewState();
@@ -160,7 +166,7 @@ bool MainWindow::init(void)
                                   << "There is a need for jack server running "
                                   << "in order to have audio output.");
         }
-        topBar = new TopBar(this, kitModel);
+        topBar = new TopBar(this, geonkickModel);
         topBar->setX(10);
         topBar->show();
         RK_ACT_BIND(this, updateGui, RK_ACT_ARGS(), topBar, updateGui());
@@ -170,10 +176,8 @@ bool MainWindow::init(void)
                     this, openFileDialog(FileDialog::Type::Save));
         RK_ACT_BIND(topBar, resetToDefault, RK_ACT_ARGS(),
                     this, resetToDefault());
-#ifndef GEONKICK_OS_WINDOWS
         RK_ACT_BIND(topBar, openExport, RK_ACT_ARGS(),
                     this, openExportDialog());
-#endif // GEONKICK_OS_WINDOWS
         RK_ACT_BIND(topBar, layerSelected,
                     RK_ACT_ARGS(GeonkickApi::Layer layer, bool b),
                     geonkickApi, enbaleLayer(layer, b));
@@ -184,7 +188,7 @@ bool MainWindow::init(void)
         envelopeWidget->setY(topBar->y() + topBar->height());
         envelopeWidget->setFixedSize(850, 305);
         envelopeWidget->show();
-        
+
         RK_ACT_BIND(this, updateGui, RK_ACT_ARGS(), envelopeWidget, updateGui());
         RK_ACT_BIND(envelopeWidget, requestUpdateGui, RK_ACT_ARGS(), this, updateGui());
         limiterWidget = new Limiter(geonkickApi, this);
@@ -192,19 +196,19 @@ bool MainWindow::init(void)
                                    envelopeWidget->y());
         RK_ACT_BIND(this, updateGui, RK_ACT_ARGS(), limiterWidget, onUpdateLimiter());
         limiterWidget->show();
-        controlAreaWidget = new ControlArea(this, kitModel, oscillators);
+        controlAreaWidget = new ControlArea(this, geonkickModel, oscillators);
         controlAreaWidget->setPosition(10, envelopeWidget->y() + envelopeWidget->height());
         controlAreaWidget->show();
         RK_ACT_BIND(this, updateGui, RK_ACT_ARGS(), controlAreaWidget, updateGui());
 
-        kitModel = controlAreaWidget->getKitModel();
-        RK_ACT_BIND(kitModel,
+        RK_ACT_BIND(geonkickModel->getKitModel(),
                     limiterUpdated,
                     RK_ACT_ARGS(KitModel::PercussionIndex index),
                     this,
                     updateLimiter(index));
         RK_ACT_BIND(limiterWidget, limiterUpdated, RK_ACT_ARGS(int val),
-                    kitModel, updatePercussion(kitModel->selectedPercussion()));
+                    geonkickModel->getKitModel(),
+                    updatePercussion(geonkickModel->getKitModel()->selectedPercussion()));
 
         RK_ACT_BIND(this, updateGui, RK_ACT_ARGS(), controlAreaWidget, updateGui());
         if (geonkickApi->isStandalone() && !presetName.empty())
@@ -215,13 +219,11 @@ bool MainWindow::init(void)
         return true;
 }
 
-#ifndef GEONKICK_OS_WINDOWS
 void MainWindow::openExportDialog()
 {
         auto w = new ExportWidget(this, geonkickApi);
         w->setPosition(30, 40);
 }
-#endif // GEONKICK_OS_WINDOWS
 
 void MainWindow::savePreset(const std::string &fileName)
 {
@@ -264,6 +266,8 @@ void MainWindow::openPreset(const std::string &fileName)
                              (std::istreambuf_iterator<char>()));
         auto state = geonkickApi->getDefaultPercussionState();
         state->loadData(fileData);
+        if (state->getName().empty() || state->getName() == "Default")
+                state->setName(filePath.stem().string());
         state->setId(geonkickApi->currentPercussion());
         geonkickApi->setPercussionState(state);
         action geonkickApi->percussionUpdated(state->getId());
@@ -313,9 +317,7 @@ void MainWindow::shortcutEvent(RkKeyEvent *event)
                         openFileDialog(FileDialog::Type::Save);
                 } else if (event->modifiers() & static_cast<int>(Rk::KeyModifiers::Control)
                            && (event->key() == Rk::Key::Key_e || event->key() == Rk::Key::Key_E)) {
-#ifndef GEONKICK_OS_WINDOWS
                         openExportDialog();
-#endif // GEONKICK_OS_WINDOWS
                 } else if ((event->modifiers() & static_cast<int>(Rk::KeyModifiers::Control))
                            && (event->key() == Rk::Key::Key_c || event->key() == Rk::Key::Key_C)) {
                         geonkickApi->copyToClipboard();
@@ -375,7 +377,7 @@ void MainWindow::dropEvent(RkDropEvent *event)
 
         std::string file = event->getFilePath();
         if (fileExtention == ".gkit" || fileExtention == ".GKIT") {
-                kitModel->open(file);
+                geonkickModel->getKitModel()->open(file);
         } else if  (fileExtention == ".gkick" || fileExtention == ".GKICK") {
                 openPreset(file);
         } else if (fileExtention == ".wav"
@@ -401,7 +403,7 @@ void MainWindow::setSample(const std::string &file)
 
 void MainWindow::updateLimiter(KitModel::PercussionIndex index)
 {
-        if (kitModel->isPercussionSelected(index))
+        if (geonkickModel->getKitModel()->isPercussionSelected(index))
                 limiterWidget->onUpdateLimiter();
 }
 
