@@ -2,7 +2,7 @@
  * File name: MainWindow.h
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -34,14 +34,15 @@ class TopBar;
 class EnvelopeWidget;
 class ControlArea;
 class Limiter;
+class GeonkickModel;
 
 class MainWindow : public GeonkickWidget
 {
  public:
-      explicit MainWindow(RkMain* app,
+      explicit MainWindow(RkMain& app,
                           GeonkickApi *api,
                           const std::string &preset = std::string());
-      explicit MainWindow(RkMain* app,
+      explicit MainWindow(RkMain& app,
                           GeonkickApi *api,
                           const RkNativeWindowInfo &info);
       ~MainWindow();
@@ -56,9 +57,7 @@ class MainWindow : public GeonkickWidget
       void openPreset(const std::string &fileName);
       void savePreset(const std::string &fileName);
       void setPreset(const std::string &fileName);
-#ifndef  GEONKICK_OS_WINDOWS
       void openExportDialog();
-#endif // GEONKICK_OS_WINDOWS
       void openPreset();
       void resetToDefault();
       void updateLimiter(KitModel::PercussionIndex index);
@@ -76,7 +75,7 @@ class MainWindow : public GeonkickWidget
       std::string presetName;
       std::string currentWorkingPath;
       Limiter *limiterWidget;
-      KitModel *kitModel;
+      GeonkickModel *geonkickModel;
 };
 
 #endif // GEONKICK_MAINWINDOW_H
