@@ -22,20 +22,29 @@
  */
 
 #include "PathListModel.h"
+#include "DesktopPaths.h"
 
 PathListModel::PathListModel(RkObject *parent)
         : RkModel(parent)
-        , pathList{"", "", ""}
 {
+        DesktopPaths desktopPaths;
+        pathList.emplace_back(desktopPaths.getHomePath());
+        pathList.emplace_back(desktopPaths.getDesktopPath());
+        pathList.emplace_back(desktopPaths.getDownloadsPath());
+        auto drives = desktopPaths.getDrivesList();
+        for (auto d: drives)
+                pathList.emplace_back(d);
+    
         stringFont.setWeight(RkFont::Weight::Bold);
         stringFont.setSize(12);
 }
 
 void PathListModel::setHomeDirectory(const std::string &path)
 {
-        pathList[0] = path;
-        pathList[1] = std::filesystem::path(path) / "Desktop";
-        pathList[2] = std::filesystem::path(path) / "Downloads";
+        if (!pathList.empty())
+                pathList[0] = path;
+        else
+                pathList.push_back(path);
         action modelChanged();
 }
 
@@ -48,7 +57,9 @@ RkVariant PathListModel::itemData(size_t index, int dataType) const
                 if (index == 0)
                         return "Home";
                 else if (pathList[index].has_filename())
-                        return pathList[index].filename();
+                        return pathList[index].filename().string();
+                else                     
+                        return pathList[index].root_name().string();
         } else if (static_cast<RkModelItem::DataType>(dataType) == RkModelItem::DataType::Size) {
                 return RkSize(0, 18);
         } else if (static_cast<RkModelItem::DataType>(dataType) == RkModelItem::DataType::Font) {
@@ -56,7 +67,7 @@ RkVariant PathListModel::itemData(size_t index, int dataType) const
         } else if (static_cast<RkModelItem::DataType>(dataType) == RkModelItem::DataType::Color) {
                 return RkColor(150, 150, 150);
         } else if (static_cast<PathListDataType>(dataType) == PathListDataType::Path) {
-                return pathList[index];
+                return pathList[index].string();
         }
         return RkVariant();
 }
@@ -74,11 +85,22 @@ int PathListModel::itemSpan() const
 void PathListModel::addPath(const std::filesystem::path &path)
 {
         auto it = std::find(pathList.begin(), pathList.end(), path);
-        if (it != pathList.end()) {
-                if (pathList.size() > 8)
-                        pathList.erase(pathList.begin() + 3);
+        if (it == pathList.end()) {
                 pathList.push_back(path);
                 action modelChanged();
         }
 }
 
+void PathListModel::removePath(const std::filesystem::path &path)
+{
+        auto it = std::find(pathList.begin(), pathList.end(), path);
+        if (it != pathList.end()) {
+                pathList.erase(it);
+                action modelChanged();
+        }
+}
+
+const std::vector<std::filesystem::path>& PathListModel::getPaths() const
+{
+        return pathList;
+}
