@@ -2,7 +2,7 @@
  * File name: audio_output.h
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iurie Nistor 
+ * Copyright (C) 2018 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -27,6 +27,7 @@
 #include "geonkick_internal.h"
 #include "ring_buffer.h"
 
+#include <pthread.h>
 #include <stdatomic.h>
 
 /**
@@ -71,13 +72,20 @@ struct gkick_audio_output
          * Specifies if the audio output is in the
          * playing state (the percussion is playing)
          */
-        _Atomic bool is_play;
+        _Atomic bool play;
 
         /**
-         * Triggers the audio thread to start to play
-         * the percussion with the maximum key velocity.
+         * Triggers the play, usualy changed from UI call.
          */
-        _Atomic bool play;
+        _Atomic bool start_play;
+
+        /**
+         * decay - note release time measured in number of audio frames.
+         * Relaxation curve for audio is liniear:
+         *   - 1.0 * (GEKICK_NOTE_RELEASE_TIME - decay) / GEKICK_NOTE_RELEASE_TIME + 1.0,
+         *    decay from GEKICK_NOTE_RELEASE_TIME to 0;
+         */
+        _Atomic int decay;
 
         /**
          * Specifies if to tune the output in accordance with
@@ -97,6 +105,9 @@ struct gkick_audio_output
         /* Output audio limiter value. */
         atomic_int limiter;
 
+        /* Enable/disable note off */
+        _Atomic bool note_off;
+
         pthread_mutex_t lock;
 };
 
@@ -113,7 +124,7 @@ gkick_audio_output_key_pressed(struct gkick_audio_output *audio_output,
                                struct gkick_note_info *key);
 
 void
-gkick_audio_add_playing_buffer_to_ring(struct gkick_audio_output *audio_output);
+gkick_audio_add_playing_buffer_to_ring(struct gkick_audio_output *audio_output, size_t size);
 
 enum geonkick_error
 gkick_audio_output_play(struct gkick_audio_output *audio_output);
@@ -122,6 +133,9 @@ gkick_audio_output_play(struct gkick_audio_output *audio_output);
 void
 gkick_audio_set_play(struct gkick_audio_output *audio_output);
 
+gkick_real
+gkick_audio_get_decay_val(struct gkick_audio_output *audio_output);
+
 gkick_real
 gkick_audio_output_tune_factor(int note_number);
 
@@ -155,4 +169,15 @@ enum geonkick_error
 gkick_audio_output_get_channel(struct gkick_audio_output *audio_output,
                                size_t *channel);
 
+void gkick_audio_get_data(struct gkick_audio_output *audio_output,
+                          gkick_real **data,
+                          gkick_real *leveler,
+                          size_t size);
+
+void gkick_audio_output_enable_note_off(struct gkick_audio_output *audio_output,
+                                 bool enable);
+
+bool gkick_audio_output_note_off(struct gkick_audio_output *audio_output);
+
+
 #endif // GKICK_AUDO_OUTPUT_H
