@@ -231,7 +231,7 @@ gkick_filter_val(struct gkick_filter *filter,
         if (isnan(in_val))
                 in_val = 0.0;
         else
-                in_val = geonkick_clamp(in_val, -1.0f, 1.0f);
+                in_val = GKICK_CLAMP(in_val, -1.0f, 1.0f);
 
         if (filter == NULL || out_val == NULL) {
                 gkick_log_error("wrong arguments");
