@@ -753,6 +753,25 @@ geonkick_set_osc_pitch_shift(struct geonkick *kick,
         return res;
 }
 
+enum geonkick_error
+geonkick_set_osc_noise_density(struct geonkick *kick,
+                               size_t osc_index,
+                               gkick_real density)
+{
+	if (kick == NULL) {
+		gkick_log_error("wrong arguments");
+		return GEONKICK_ERROR;
+	}
+
+        enum geonkick_error res;
+        res = gkick_synth_set_osc_noise_density(kick->synths[kick->per_index],
+                                                osc_index,
+                                                density);
+        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
+                geonkick_wakeup(kick);
+        return res;
+}
+
 enum geonkick_error
 geonkick_set_osc_amplitude(struct geonkick *kick,
                            size_t osc_index,
@@ -808,6 +827,18 @@ geonkick_get_osc_pitch_shift(struct geonkick *kick,
                                                semitones);
 }
 
+enum geonkick_error
+geonkick_get_osc_noise_density(struct geonkick *kick,
+                               size_t osc_index,
+                               gkick_real *density)
+{
+	if (kick == NULL || density == NULL)
+		return GEONKICK_ERROR;
+	return gkick_synth_get_osc_noise_density(kick->synths[kick->per_index],
+                                                 osc_index,
+                                                 density);
+}
+
 enum geonkick_error
 geonkick_play(struct geonkick *kick, size_t id)
 {
@@ -1104,203 +1135,6 @@ geonkick_audio_process(struct geonkick *kick,
                                    size);
 }
 
-enum geonkick_error
-geonkick_compressor_enable(struct geonkick *kick,
-                           int enable)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_enable(kick->synths[kick->per_index],
-                                            enable);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-
-}
-
-enum geonkick_error
-geonkick_compressor_is_enabled(struct geonkick *kick,
-                               int *enabled)
-{
-        if (kick == NULL || enabled == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_is_enabled(kick->synths[kick->per_index],
-                                                 enabled);
-}
-
-enum geonkick_error
-geonkick_compressor_set_attack(struct geonkick *kick,
-                               gkick_real attack)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_set_attack(kick->synths[kick->per_index],
-                                                attack);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-}
-
-enum geonkick_error
-geonkick_compressor_get_attack(struct geonkick *kick,
-                               gkick_real *attack)
-{
-        if (kick == NULL || attack == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_get_attack(kick->synths[kick->per_index],
-                                                 attack);
-}
-
-enum geonkick_error
-geonkick_compressor_set_release(struct geonkick *kick,
-                                gkick_real release)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_set_release(kick->synths[kick->per_index],
-                                                 release);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-}
-
-enum geonkick_error
-geonkick_compressor_get_release(struct geonkick *kick,
-                                gkick_real *release)
-{
-        if (kick == NULL || release == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_get_release(kick->synths[kick->per_index],
-                                                  release);
-}
-
-enum geonkick_error
-geonkick_compressor_set_threshold(struct geonkick *kick,
-                                  gkick_real threshold)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_set_threshold(kick->synths[kick->per_index],
-                                                   threshold);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-}
-
-enum geonkick_error
-geonkick_compressor_get_threshold(struct geonkick *kick,
-                                  gkick_real *threshold)
-{
-        if (kick == NULL || threshold == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_get_threshold(kick->synths[kick->per_index],
-                                                    threshold);
-}
-
-enum geonkick_error
-geonkick_compressor_set_ratio(struct geonkick *kick,
-                              gkick_real ratio)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_set_ratio(kick->synths[kick->per_index],
-                                               ratio);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-}
-
-enum geonkick_error
-geonkick_compressor_get_ratio(struct geonkick *kick,
-                              gkick_real *ratio)
-{
-        if (kick == NULL || ratio == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_get_ratio(kick->synths[kick->per_index],
-                                                ratio);
-}
-
-enum geonkick_error
-geonkick_compressor_set_knee(struct geonkick *kick,
-                             gkick_real knee)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_set_knee(kick->synths[kick->per_index],
-                                              knee);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-}
-
-enum geonkick_error
-geonkick_compressor_get_knee(struct geonkick *kick,
-                             gkick_real *knee)
-{
-        if (kick == NULL || knee == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_get_knee(kick->synths[kick->per_index], knee);
-}
-
-enum geonkick_error
-geonkick_compressor_set_makeup(struct geonkick *kick,
-                               gkick_real makeup)
-{
-        if (kick == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        enum geonkick_error res;
-        res = gkick_synth_compressor_set_makeup(kick->synths[kick->per_index],
-                                                makeup);
-        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
-                geonkick_wakeup(kick);
-        return res;
-
-}
-
-enum geonkick_error
-geonkick_compressor_get_makeup(struct geonkick *kick,
-                               gkick_real *makeup)
-{
-        if (kick == NULL || makeup == NULL) {
-                gkick_log_error("wrong arguments");
-                return GEONKICK_ERROR;
-        }
-        return gkick_synth_compressor_get_makeup(kick->synths[kick->per_index],
-                                                 makeup);
-}
-
 enum geonkick_error
 geonkick_distortion_enable(struct geonkick *kick, int enable)
 {
@@ -1936,6 +1770,22 @@ geonkick_get_sample_preview_limiter(struct geonkick *kick, gkick_real *val)
                                            val);
 }
 
+enum geonkick_error
+geonkick_percussion_enable_note_off(struct geonkick *kick,
+                                    size_t id,
+                                    bool enable)
+{
+        return gkick_mixer_enable_note_off(kick->audio->mixer, id, enable);
+}
+
+enum geonkick_error
+geonkick_percussion_note_off_enabled(struct geonkick *kick,
+                                     size_t id,
+                                     bool *enabled)
+{
+        return gkick_mixer_note_off_enabled(kick->audio->mixer, id, enabled);
+}
+
 void
 geonkick_wakeup(struct geonkick *kick)
 {
@@ -1967,3 +1817,7 @@ int geonkick_rand(unsigned int *seed)
 #endif // GEONKICK_OS_GNU
 }
 
+size_t geonkick_layers_number(void)
+{
+        return GKICK_OSC_GROUPS_NUMBER;
+}
