@@ -2,7 +2,7 @@
  * File name: geonkick.h
  * Project: GeonKick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of GeonKick.
  *
@@ -30,7 +30,6 @@ extern "C" {
 
 #include <stdio.h>
 #include <errno.h>
-//#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
@@ -62,6 +61,10 @@ typedef float gkick_real;
 #define GEONKICK_APP_NAME "geonkick"
 #define GEOKICK_VERSION_STRING GEONKICK_VERSION_STR
 
+#if defined(GEONKICK_LIMITED_VERSION) && !defined(GEONKICK_SINGLE)
+#define GEONKICK_SINGLE
+#endif // GEONKICK_LIMITED_VERSION
+
 #ifdef GEONKICK_AUDIO_JACK
 #define GKICK_IS_STANDALONE (1)
 #else
@@ -91,6 +94,7 @@ enum geonkick_envelope_type {
         GEONKICK_DISTORTION_VOLUME_ENVELOPE = 4,
         GEONKICK_PITCH_SHIFT_ENVELOPE       = 5,
 	GEONKICK_FILTER_Q_ENVELOPE          = 6,
+        GEONKICK_NOISE_DENSITY_ENVELOPE     = 7
 };
 
 enum gkick_filter_type {
@@ -115,7 +119,11 @@ enum GEONKICK_MODULE {
         GEONKICK_MODULE_JACK    = 1
 };
 
+#ifdef GEONKICK_LIMITED_VERSION
+#define GKICK_OSC_GROUPS_NUMBER 1
+#else
 #define GKICK_OSC_GROUPS_NUMBER 3
+#endif // GEONKICK_LIMITED_VERSION
 #define GKICK_OSC_GROUP_SIZE 3
 
 enum geonkick_channel_type {
@@ -367,6 +375,11 @@ geonkick_set_osc_pitch_shift(struct geonkick *kick,
                            size_t osc_index,
                            gkick_real semitones);
 
+enum geonkick_error
+geonkick_set_osc_noise_density(struct geonkick *kick,
+                               size_t osc_index,
+                               gkick_real density);
+
 enum geonkick_error
 geonkick_get_osc_amplitude(struct geonkick *kick,
                            size_t osc_index,
@@ -381,6 +394,11 @@ geonkick_get_osc_pitch_shift(struct geonkick *kick,
                              size_t osc_index,
                              gkick_real *semitones);
 
+enum geonkick_error
+geonkick_get_osc_noise_density(struct geonkick *kick,
+                               size_t osc_index,
+                               gkick_real *density);
+
 enum geonkick_error
 geonkick_play(struct geonkick *kick, size_t id);
 
@@ -478,62 +496,6 @@ geonkick_audio_process(struct geonkick *kick,
                        size_t channel,
                        size_t size);
 
-enum geonkick_error
-geonkick_compressor_enable(struct geonkick *kick,
-                           int enable);
-
-enum geonkick_error
-geonkick_compressor_is_enabled(struct geonkick *kick,
-                               int *enabled);
-
-enum geonkick_error
-geonkick_compressor_set_attack(struct geonkick *kick,
-                               gkick_real attack);
-
-enum geonkick_error
-geonkick_compressor_get_attack(struct geonkick *kick,
-                               gkick_real *attack);
-
-enum geonkick_error
-geonkick_compressor_set_release(struct geonkick *kick,
-                                gkick_real release);
-
-enum geonkick_error
-geonkick_compressor_get_release(struct geonkick *kick,
-                                gkick_real *release);
-
-enum geonkick_error
-geonkick_compressor_set_threshold(struct geonkick *kick,
-                                  gkick_real threshold);
-
-enum geonkick_error
-geonkick_compressor_get_threshold(struct geonkick *kick,
-                                  gkick_real *threshold);
-
-enum geonkick_error
-geonkick_compressor_set_ratio(struct geonkick *kick,
-                              gkick_real ratio);
-
-enum geonkick_error
-geonkick_compressor_get_ratio(struct geonkick *kick,
-                              gkick_real *ratio);
-
-enum geonkick_error
-geonkick_compressor_set_knee(struct geonkick *kick,
-                             gkick_real knee);
-
-enum geonkick_error
-geonkick_compressor_get_knee(struct geonkick *kick,
-                             gkick_real *knee);
-
-enum geonkick_error
-geonkick_compressor_set_makeup(struct geonkick *kick,
-                               gkick_real makeup);
-
-enum geonkick_error
-geonkick_compressor_get_makeup(struct geonkick *kick,
-                               gkick_real *makeup);
-
 enum geonkick_error
 geonkick_distortion_enable(struct geonkick *kick,
                            int enable);
@@ -662,7 +624,7 @@ enum geonkick_error
 geonkick_ged_forced_midi_channel(struct geonkick *kick,
                                  signed char *channel,
                                  bool *force);
-        
+
 size_t
 geonkick_midi_channels_number();
 
@@ -740,6 +702,23 @@ geonkick_set_sample_preview_limiter(struct geonkick *kick, gkick_real val);
 enum geonkick_error
 geonkick_get_sample_preview_limiter(struct geonkick *kick, gkick_real *val);
 
+enum geonkick_error
+geonkick_percussion_enable_note_off(struct geonkick *kick,
+                                    size_t id,
+                                    bool enable);
+
+enum geonkick_error
+geonkick_percussion_note_off_enabled(struct geonkick *kick,
+                                     size_t id,
+                                     bool *enabled);
+
+enum geonkick_error
+geonkick_note_off_enabled(struct geonkick *kick,
+                          size_t id,
+                          bool *enabled);
+
+size_t geonkick_layers_number(void);
+
 bool
 geonkick_is_plugin();
 
