@@ -2,7 +2,7 @@
  * File name: gkick_buffer.c
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iurie Nistor 
+ * Copyright (C) 2018 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -125,20 +125,17 @@ gkick_buffer_stretch_get_next(struct gkick_buffer *buffer,
                 return 0.0f;
         } else if (buffer->size == 1) {
                 return buffer->buff[0];
-        } else if (buffer->size > 1) {
-                if (buffer->currentIndex <= buffer->size - 2) {
-                        /* Do linear interpolation. */
-                        gkick_real d = buffer->floatIndex - buffer->currentIndex;
-                        gkick_real val = buffer->buff[buffer->currentIndex] * (1.0f - d)
-                                + buffer->buff[buffer->currentIndex + 1] * d;
-                        buffer->floatIndex += factor;
-                        buffer->currentIndex = (size_t)buffer->floatIndex;
-                        return val;
-                } else if (buffer->currentIndex == buffer->size - 1) {
-                        float val = buffer->buff[buffer->currentIndex++];
-                        buffer->floatIndex = buffer->currentIndex;
-                        return val;
-                }
+        } else if (buffer->currentIndex >= 0 && buffer->currentIndex < buffer->size - 1) {
+                gkick_real d = buffer->floatIndex - buffer->currentIndex;
+                gkick_real val = (buffer->buff[buffer->currentIndex + 1] - buffer->buff[buffer->currentIndex]) * d
+                        + buffer->buff[buffer->currentIndex];
+                buffer->floatIndex += factor;
+                buffer->currentIndex = (size_t)buffer->floatIndex;
+                return val;
+        } else if (buffer->currentIndex == buffer->size - 1) {
+                float val = buffer->buff[buffer->currentIndex++];
+                buffer->floatIndex = buffer->currentIndex;
+                return val;
         }
         return 0.0f;
 }
@@ -182,5 +179,5 @@ gkick_buffer_push_back(struct gkick_buffer *buffer,
 bool
 gkick_buffer_is_end(struct gkick_buffer *buffer)
 {
-        return (buffer->size < 1) || (buffer->currentIndex > buffer->size - 1);
+        return (buffer->size < 1) || (buffer->currentIndex > (buffer->size - 1));
 }
