@@ -2,7 +2,7 @@
  * File name: synthesizer.c
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iurie Nistor 
+ * Copyright (C) 2018 Iurie Nistor
  *
  * This file is part of GeonKick.
  *
@@ -56,24 +56,12 @@ gkick_synth_new(struct gkick_synth **synth, int sample_rate)
         }
         (*synth)->filter_enabled = 0;
 
-        if (gkick_compressor_new(&(*synth)->compressor, (*synth)->sample_rate) != GEONKICK_OK) {
-                gkick_log_error("can't create compressor");
-                gkick_synth_free(synth);
-                return GEONKICK_ERROR;
-        }
-
         if (gkick_distortion_new(&(*synth)->distortion, (*synth)->sample_rate) != GEONKICK_OK) {
                 gkick_log_error("can't create distortion");
                 gkick_synth_free(synth);
                 return GEONKICK_ERROR;
         }
 
-        if (gkick_filter_new(&(*synth)->filter, (*synth)->sample_rate) != GEONKICK_OK) {
-                gkick_log_error("can't create filter");
-                gkick_synth_free(synth);
-		return GEONKICK_ERROR;
-        }
-
         (*synth)->envelope = gkick_envelope_create();
         if ((*synth)->envelope == NULL) {
                 gkick_log_error("can't create envelope");
@@ -119,9 +107,6 @@ void gkick_synth_free(struct gkick_synth **synth)
                         if ((*synth)->filter)
                                 gkick_filter_free(&(*synth)->filter);
 
-                        if ((*synth)->compressor)
-                                gkick_compressor_free(&(*synth)->compressor);
-
                         if ((*synth)->distortion)
                                 gkick_distortion_free(&(*synth)->distortion);
 
@@ -494,7 +479,7 @@ gkick_synth_osc_env_update_point(struct gkick_synth *synth,
 }
 
 enum geonkick_error
-synth_osc_env_set_apply_type(struct gkick_synth *synth, 
+synth_osc_env_set_apply_type(struct gkick_synth *synth,
 			     size_t osc_index,
 			     size_t env_index,
 			     enum gkick_envelope_apply_type apply_type)
@@ -1061,7 +1046,7 @@ gkick_synth_kick_update_env_point(struct gkick_synth *synth,
 }
 
 enum geonkick_error
-synth_kick_env_set_apply_type(struct gkick_synth *synth, 
+synth_kick_env_set_apply_type(struct gkick_synth *synth,
 			      enum geonkick_envelope_type env_type,
 			      enum gkick_envelope_apply_type apply_type)
 {
@@ -1079,7 +1064,7 @@ synth_kick_env_set_apply_type(struct gkick_synth *synth,
                 synth->buffer_update = true;
         }
         gkick_synth_unlock(synth);
-        return GEONKICK_OK;	
+        return GEONKICK_OK;
 }
 
 enum geonkick_error
@@ -1097,7 +1082,7 @@ synth_kick_env_get_apply_type(struct gkick_synth *synth,
                 break;
         }
         gkick_synth_unlock(synth);
-        return GEONKICK_OK;	
+        return GEONKICK_OK;
 }
 
 enum geonkick_error
@@ -1202,6 +1187,57 @@ gkick_synth_get_osc_pitch_shift(struct gkick_synth *synth,
 	return GEONKICK_OK;
 }
 
+enum geonkick_error
+gkick_synth_set_osc_noise_density(struct gkick_synth *synth,
+                                size_t osc_index,
+                                gkick_real density)
+{
+        struct gkick_oscillator* osc;
+
+	if (synth == NULL) {
+                gkick_log_error("wrong arguments");
+		return GEONKICK_ERROR;
+	}
+
+	gkick_synth_lock(synth);
+	osc = gkick_synth_get_oscillator(synth, osc_index);
+	if (osc == NULL) {
+		gkick_log_error("can't get oscillator");
+		gkick_synth_unlock(synth);
+		return GEONKICK_ERROR;
+	}
+	osc->noise_density = density;
+        if (synth->osc_groups[osc_index / GKICK_OSC_GROUP_SIZE]
+            && osc->state == GEONKICK_OSC_STATE_ENABLED) {
+                synth->buffer_update = true;
+        }
+
+	gkick_synth_unlock(synth);
+	return GEONKICK_OK;
+}
+
+enum geonkick_error
+gkick_synth_get_osc_noise_density(struct gkick_synth *synth,
+                                size_t osc_index,
+                                gkick_real *density)
+{
+	if (synth == NULL || density == NULL) {
+                gkick_log_error("wrong arguments");
+		return GEONKICK_ERROR;
+	}
+
+	gkick_synth_lock(synth);
+	struct gkick_oscillator* osc = gkick_synth_get_oscillator(synth, osc_index);
+	if (osc == NULL) {
+		gkick_log_error("can't get oscillator");
+		gkick_synth_unlock(synth);
+		return GEONKICK_ERROR;
+	}
+	*density = osc->noise_density;
+	gkick_synth_unlock(synth);
+	return GEONKICK_OK;
+}
+
 enum geonkick_error
 gkick_synth_set_osc_amplitude(struct gkick_synth *synth,
                               size_t osc_index,
@@ -1414,10 +1450,6 @@ gkick_synth_get_value(struct gkick_synth *synth,
         if (enabled)
                 gkick_distortion_val(synth->distortion, val, &val, env_x);
 
-        gkick_compressor_is_enabled(synth->compressor, &enabled);
-        if (enabled)
-                gkick_compressor_val(synth->compressor, val, &val);
-
         return val;
 }
 
@@ -1667,154 +1699,6 @@ gkick_synth_osc_is_enabled_filter(struct gkick_synth *synth,
         return GEONKICK_OK;
 }
 
-enum geonkick_error
-gkick_synth_compressor_enable(struct gkick_synth *synth,
-                              int enable)
-{
-        synth->buffer_update = true;
-        return gkick_compressor_enable(synth->compressor,
-                                       enable);
-}
-
-enum geonkick_error
-gkick_synth_compressor_is_enabled(struct gkick_synth *synth,
-                                  int *enabled)
-{
-        return gkick_compressor_is_enabled(synth->compressor
-                                           , enabled);
-}
-
-enum geonkick_error
-gkick_synth_compressor_set_attack(struct gkick_synth *synth,
-                                  gkick_real attack)
-{
-        enum geonkick_error res;
-        res = gkick_compressor_set_attack(synth->compressor,
-                                          attack);
-	int enabled  = 0;
-        gkick_compressor_is_enabled(synth->compressor,
-                                    &enabled);
-        if (res == GEONKICK_OK && enabled)
-                synth->buffer_update = true;
-        return res;
-}
-
-enum geonkick_error
-gkick_synth_compressor_get_attack(struct gkick_synth *synth,
-                                  gkick_real *attack)
-{
-        return gkick_compressor_get_attack(synth->compressor,
-                                           attack);
-}
-
-enum geonkick_error
-gkick_synth_compressor_set_release(struct gkick_synth *synth,
-                                   gkick_real release)
-{
-        enum geonkick_error res;
-        res = gkick_compressor_set_release(synth->compressor,
-                                           release);
-	int enabled = 0;
-        gkick_compressor_is_enabled(synth->compressor,
-                                    &enabled);
-        if (res == GEONKICK_OK && enabled)
-                synth->buffer_update = true;
-        return res;
-}
-
-enum geonkick_error
-gkick_synth_compressor_get_release(struct gkick_synth *synth,
-                                   gkick_real *release)
-{
-        return gkick_compressor_get_release(synth->compressor,
-                                            release);
-}
-
-enum geonkick_error
-gkick_synth_compressor_set_threshold(struct gkick_synth *synth,
-                                     gkick_real threshold)
-{
-	enum geonkick_error res;
-        res = gkick_compressor_set_threshold(synth->compressor,
-                                             threshold);
-        int enabled = 0;
-        gkick_compressor_is_enabled(synth->compressor,
-                                    &enabled);
-        if (res == GEONKICK_OK && enabled)
-                synth->buffer_update = true;
-        return res;
-}
-
-enum geonkick_error
-gkick_synth_compressor_get_threshold(struct gkick_synth *synth,
-                                     gkick_real *threshold)
-{
-        return gkick_compressor_get_threshold(synth->compressor,
-                                              threshold);
-}
-
-enum geonkick_error
-gkick_synth_compressor_set_ratio(struct gkick_synth *synth,
-                                 gkick_real ratio)
-{
-        enum geonkick_error res;
-        res = gkick_compressor_set_ratio(synth->compressor, ratio);
-        int enabled = 0;
-        gkick_compressor_is_enabled(synth->compressor, &enabled);
-        if (res == GEONKICK_OK && enabled)
-                synth->buffer_update = true;
-        return res;
-}
-
-enum geonkick_error
-gkick_synth_compressor_get_ratio(struct gkick_synth *synth,
-                                 gkick_real *ratio)
-{
-        return gkick_compressor_get_ratio(synth->compressor,
-                                          ratio);
-}
-
-enum geonkick_error
-gkick_synth_compressor_set_knee(struct gkick_synth *synth,
-                                gkick_real knee)
-{
-        enum geonkick_error res;
-        res = gkick_compressor_set_knee(synth->compressor, knee);
-        int enabled = false;
-        gkick_compressor_is_enabled(synth->compressor, &enabled);
-        if (res == GEONKICK_OK && enabled)
-                synth->buffer_update = true;
-        return res;
-}
-
-enum geonkick_error
-gkick_synth_compressor_get_knee(struct gkick_synth *synth,
-                                gkick_real *knee)
-{
-        return gkick_compressor_get_threshold(synth->compressor, knee);
-}
-
-enum geonkick_error
-gkick_synth_compressor_set_makeup(struct gkick_synth *synth,
-                                  gkick_real makeup)
-{
-        enum geonkick_error res;
-        res = gkick_compressor_set_makeup(synth->compressor, makeup);
-        int enabled;
-        gkick_compressor_is_enabled(synth->compressor, &enabled);
-        if (res == GEONKICK_OK && enabled)
-                synth->buffer_update = true;
-        return res;
-}
-
-enum geonkick_error
-gkick_synth_compressor_get_makeup(struct gkick_synth *synth,
-                                  gkick_real *makeup)
-{
-        return gkick_compressor_get_makeup(synth->compressor,
-                                           makeup);
-}
-
 enum geonkick_error
 gkick_synth_distortion_enable(struct gkick_synth *synth,
                               int enable)
