@@ -25,7 +25,6 @@
 #define GKICK_SYNTHESIZER_H
 
 #include "geonkick_internal.h"
-#include "compressor.h"
 #include "distortion.h"
 #include "audio_output.h"
 
@@ -61,9 +60,6 @@ struct gkick_synth {
         struct gkick_filter *filter;
         int filter_enabled;
 
-        /* Kick compressor */
-        struct gkick_compressor *compressor;
-
         /* Kick distortion */
         struct gkick_distortion *distortion;
 
@@ -331,6 +327,16 @@ gkick_synth_get_osc_pitch_shift(struct gkick_synth *synth,
                                 size_t osc_index,
                                 gkick_real *semitones);
 
+enum geonkick_error
+gkick_synth_set_osc_noise_density(struct gkick_synth *synth,
+                                size_t osc_index,
+                                gkick_real density);
+
+enum geonkick_error
+gkick_synth_get_osc_noise_density(struct gkick_synth *synth,
+                                size_t osc_index,
+                                gkick_real *density);
+
 enum geonkick_error
 gkick_synth_set_osc_amplitude(struct gkick_synth *synth,
                               size_t osc_index,
@@ -406,62 +412,6 @@ gkick_synth_osc_is_enabled_filter(struct gkick_synth *synth,
                                   size_t osc_index,
                                   int *enabled);
 
-enum geonkick_error
-gkick_synth_compressor_enable(struct gkick_synth *synth,
-			      int enable);
-
-enum geonkick_error
-gkick_synth_compressor_is_enabled(struct gkick_synth *synth,
-				  int *enabled);
-
-enum geonkick_error
-gkick_synth_compressor_set_attack(struct gkick_synth *synth,
-				  gkick_real attack);
-
-enum geonkick_error
-gkick_synth_compressor_get_attack(struct gkick_synth *synth,
-				  gkick_real *attack);
-
-enum geonkick_error
-gkick_synth_compressor_set_release(struct gkick_synth *synth,
-				   gkick_real release);
-
-enum geonkick_error
-gkick_synth_compressor_get_release(struct gkick_synth *synth,
-				   gkick_real *release);
-
-enum geonkick_error
-gkick_synth_compressor_set_threshold(struct gkick_synth *synth,
-				     gkick_real threshold);
-
-enum geonkick_error
-gkick_synth_compressor_get_threshold(struct gkick_synth *synth,
-				     gkick_real *threshold);
-
-enum geonkick_error
-gkick_synth_compressor_set_ratio(struct gkick_synth *synth,
-				 gkick_real ratio);
-
-enum geonkick_error
-gkick_synth_compressor_get_ratio(struct gkick_synth *synth,
-				 gkick_real *ratio);
-
-enum geonkick_error
-gkick_synth_compressor_set_knee(struct gkick_synth *synth,
-				gkick_real knee);
-
-enum geonkick_error
-gkick_synth_compressor_get_knee(struct gkick_synth *synth,
-				gkick_real *knee);
-
-enum geonkick_error
-gkick_synth_compressor_set_makeup(struct gkick_synth *synth,
-				  gkick_real makeup);
-
-enum geonkick_error
-gkick_synth_compressor_get_makeup(struct gkick_synth *synth,
-				  gkick_real *makeup);
-
 enum geonkick_error
 gkick_synth_distortion_enable(struct gkick_synth *synth,
 			      int enable);
