@@ -2,7 +2,7 @@
  * File name: envelope_widget.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -44,9 +44,11 @@ EnvelopeWidget::EnvelopeWidget(GeonkickWidget *parent,
                                const std::vector<std::unique_ptr<Oscillator>> &oscillators)
         : GeonkickWidget(parent)
         , drawArea{nullptr}
+#ifndef GEONKICK_LIMITED_VERSION
         , layer1Button{nullptr}
         , layer2Button{nullptr}
         , layer3Button{nullptr}
+#endif // GEONKICK_LIMITED_VERSION
         , geonkickApi{api}
 {
         // Create drawing area.
@@ -67,11 +69,11 @@ EnvelopeWidget::EnvelopeWidget(GeonkickWidget *parent,
         envelopes.insert({static_cast<int>(Envelope::Category::Oscillator2),
                         std::move(envelope)});
 
-        // Noise envelope
-        oscillator = oscillators[static_cast<int>(Oscillator::Type::Noise)].get();
+        // Oscillator3 envelope
+        oscillator = oscillators[static_cast<int>(Oscillator::Type::Oscillator3)].get();
         envelope = std::make_unique<OscillatorEnvelope>(oscillator, rect);
-        envelope->setCategory(Envelope::Category::Noise);
-        envelopes.insert({static_cast<int>(Envelope::Category::Noise),
+        envelope->setCategory(Envelope::Category::Oscillator3);
+        envelopes.insert({static_cast<int>(Envelope::Category::Oscillator3),
                         std::move(envelope)});
 
         // General envelope
@@ -79,16 +81,20 @@ EnvelopeWidget::EnvelopeWidget(GeonkickWidget *parent,
         generalEnvelope->setCategory(Envelope::Category::General);
         envelopes.insert({static_cast<int>(Envelope::Category::General),
                         std::move(generalEnvelope)});
+#ifndef GEONKICK_LIMITED_VERSION
         createButtomMenu();
+#endif // GEONKICK_LIMITED_VERSION
         showEnvelope(Envelope::Category::General, Envelope::Type::Amplitude);
         RK_ACT_BIND(viewState(), envelopeChanged,
                     RK_ACT_ARGS(Envelope::Category category, Envelope::Type envelope),
                     this, showEnvelope(category, envelope));
-        
+
+        createZoomInfoLabel();
         createPointInfoLabel();
         drawArea->show();
 }
 
+#ifndef GEONKICK_LIMITED_VERSION
 void EnvelopeWidget::createButtomMenu()
 {
         auto buttomAreaWidget = new GeonkickWidget(drawArea);
@@ -97,15 +103,16 @@ void EnvelopeWidget::createButtomMenu()
         buttomAreaWidget->setPosition(55 + drawArea->x(),
                                       drawArea->y() + drawArea->height() - buttomAreaWidget->height() - 6);
 
+        auto menuContainer = new RkContainer(buttomAreaWidget);
         createLayersButtons(buttomAreaWidget);
-	menuContainer = new RkContainer(buttomAreaWidget);
-	menuContainer->addWidget(layer1Button);
+        menuContainer->addWidget(layer1Button);
         menuContainer->addSpace(5);
         menuContainer->addWidget(layer2Button);
         menuContainer->addSpace(5);
         menuContainer->addWidget(layer3Button);
         buttomAreaWidget->show();
 }
+#endif // GEONKICK_LIMITED_VERSION
 
 void EnvelopeWidget::createPointInfoLabel()
 {
@@ -120,6 +127,19 @@ void EnvelopeWidget::createPointInfoLabel()
                     pointInfoLabel, setText(info));
 }
 
+void EnvelopeWidget::createZoomInfoLabel()
+{
+        auto zoomInfo = "1:" + Geonkick::doubleToStr(drawArea->getEnvelope()->getZoom(), 0);
+        auto zoomInfoLabel = new RkLabel(drawArea, zoomInfo);
+        zoomInfoLabel->setBackgroundColor(drawArea->background());
+        zoomInfoLabel->setTextColor({180, 180, 180});
+        zoomInfoLabel->setFixedSize(30, 16);
+        zoomInfoLabel->setPosition(10, drawArea->height() - zoomInfoLabel->height() - 8);
+        RK_ACT_BIND(drawArea, zoomUpdated,
+                    RK_ACT_ARGS(const std::string &info),
+                    zoomInfoLabel, setText("1:" + info));
+}
+
 void EnvelopeWidget::updateKickGraph(std::shared_ptr<RkImage> graphImage)
 {
         if (graphImage && !graphImage->isNull())
@@ -146,6 +166,7 @@ void EnvelopeWidget::hideEnvelope(bool b)
         drawArea->setHideEnvelope(b);
 }
 
+#ifndef GEONKICK_LIMITED_VERSION
 void EnvelopeWidget::createLayersButtons(GeonkickWidget *buttomAreaWidget)
 {
         layer1Button = new GeonkickButton(buttomAreaWidget);
@@ -188,12 +209,15 @@ void EnvelopeWidget::createLayersButtons(GeonkickWidget *buttomAreaWidget)
         RK_ACT_BIND(layer3Button, toggled, RK_ACT_ARGS(bool b),
                     this, setLayer(GeonkickApi::Layer::Layer3));
 }
+#endif // GEONKICK_LIMITED_VERSION
 
 void EnvelopeWidget::setLayer(GeonkickApi::Layer layer)
 {
+#ifndef GEONKICK_LIMITED_VERSION
         layer1Button->setPressed(GeonkickApi::Layer::Layer1 == layer);
         layer2Button->setPressed(GeonkickApi::Layer::Layer2 == layer);
         layer3Button->setPressed(GeonkickApi::Layer::Layer3 == layer);
+#endif // GEONKICK_LIMITED_VERSION
         geonkickApi->setLayer(layer);
         action requestUpdateGui();
 }
