@@ -27,17 +27,13 @@
 #include "file_dialog.h"
 
 #include <RkLabel.h>
-#include <RkLineEdit.h>
-#include <RkProgressBar.h>
+#include <RkContainer.h>
 
 #include <sndfile.h>
 
 RK_DECLARE_IMAGE_RC(export_bk);
 RK_DECLARE_IMAGE_RC(export_format_unpressed);
 RK_DECLARE_IMAGE_RC(export_format_pressed);
-RK_DECLARE_IMAGE_RC(export_browse);
-RK_DECLARE_IMAGE_RC(export_button);
-RK_DECLARE_IMAGE_RC(export_cancel);
 RK_DECLARE_IMAGE_RC(flac16_radio_button);
 RK_DECLARE_IMAGE_RC(flac16_radio_button_active);
 RK_DECLARE_IMAGE_RC(flac24_radio_button);
@@ -56,11 +52,8 @@ RK_DECLARE_IMAGE_RC(stereo_radio_button);
 RK_DECLARE_IMAGE_RC(stereo_radio_button_active);
 
 ExportWidget::ExportWidget(GeonkickWidget *parent, GeonkickApi *api)
-        : GeonkickWidget(parent, Rk::WindowFlags::Dialog)
+        : GeonkickWidget(parent, Rk::WidgetFlags::Popup)
         , geonkickApi{api}
-        , locationEdit{nullptr}
-        , fileNameEdit{nullptr}
-        , browseLocation{nullptr}
         , flac16Button{nullptr}
         , flac24Button{nullptr}
         , wav16Button{nullptr}
@@ -69,16 +62,25 @@ ExportWidget::ExportWidget(GeonkickWidget *parent, GeonkickApi *api)
         , oggButton{nullptr}
         , monoButton{nullptr}
         , stereoButton{nullptr}
-        , progressBar{nullptr}
-        , exportButton{nullptr}
-        , cancelButton{nullptr}
-        , errorLabel{nullptr}
         , selectedFormat{ExportFormat::Wav16}
         , channelsType{ChannelsType::Mono}
 {
+        setFixedSize(600, 410);
         setPosition(30, 40);
-        setBorderWidth(1);
-        setBorderColor(40, 40, 40);
+        setBorderWidth(2);
+        setBorderColor(80, 80, 80);
+
+        auto fileDialog = new FileDialog(this, FileDialog::Type::Save,
+                                         Rk::WidgetFlags::Widget,
+                                         "Select Path - " + std::string(GEONKICK_NAME));
+        fileDialog->setFilters({".flac", ".wav", ".ogg", ".FLAC", ".WAV", ".OGG"});
+        fileDialog->setPosition(0, 40);
+        fileDialog->setHomeDirectory(geonkickApi->getSettings("GEONKICK_CONFIG/HOME_PATH"));
+        fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("ExportDialog/Location").string());
+        RK_ACT_BIND(fileDialog, selectedFile,
+                    RK_ACT_ARGS(const std::string &file), this,
+                    exportInstrument(file));
+        RK_ACT_BIND(fileDialog, rejected, RK_ACT_ARGS(), this, close());
 
         std::string format = geonkickApi->getSettings("ExportDialog/Format");
         if (!format.empty())
@@ -88,63 +90,9 @@ ExportWidget::ExportWidget(GeonkickWidget *parent, GeonkickApi *api)
         if (!channel.empty())
                 channelsType = static_cast<ChannelsType>(std::stoi(channel));
 
-        setFixedSize(521, 184);
         setTitle("Export - " + std::string(GEONKICK_NAME));
-        setBackgroundImage(RkImage(521, 184, RK_IMAGE_RC(export_bk)));
-
-        errorLabel = new RkLabel(this);
-        errorLabel->setFixedSize(150, 15);
-        errorLabel->setBackgroundColor(background());
-        errorLabel->setPosition(width() - 180, 105);
-        errorLabel->setTextColor({200, 0, 0});
-        errorLabel->hide();
-
-        locationEdit = new RkLineEdit(this);
-        locationEdit->setFont(font());
-        locationEdit->setTitle("locationEdit");
-        locationEdit->show();
-        locationEdit->setText(geonkickApi->currentWorkingPath("ExportDialog/Location").string());
-        locationEdit->setSize(120, 25);
-        locationEdit->setPosition(82, 54);
-        RK_ACT_BIND(locationEdit, textEdited, RK_ACT_ARGS(const std::string& text), this, resetProgressBar());
-
-        browseLocation = new GeonkickButton(this);
-        browseLocation->setCheckable(true);
-        browseLocation->setUnpressedImage(RkImage(90, 30, RK_IMAGE_RC(export_browse)));
-        browseLocation->setPosition(205, 52);
-        RK_ACT_BIND(browseLocation, toggled, RK_ACT_ARGS(bool b), this, browse());
-
-        fileNameEdit = new RkLineEdit(this);
-        fileNameEdit->setFont(font());
-        fileNameEdit->show();
-        fileNameEdit->setFixedSize(100, 25);
-        fileNameEdit->setPosition(380, 54);
-        fileNameEdit->setText(geonkickApi->getSettings("ExportDialog/FileName"));
-        RK_ACT_BIND(fileNameEdit, textEdited, RK_ACT_ARGS(const std::string& text), this, resetProgressBar());
-        RK_ACT_BIND(fileNameEdit, textEdited, RK_ACT_ARGS(const std::string& text), this, showError());
 
         createFormatButtons();
-        createChannelsButtons();
-
-        progressBar = new RkProgressBar(this);
-        progressBar->setFixedSize(width() - 50, 4);
-        progressBar->setPosition(25, 122);
-        progressBar->setRange(0, 100);
-        progressBar->setBorderWidth(1);
-        progressBar->setBorderColor(40, 40, 40);
-        progressBar->setBackgroundColor(background());
-
-        exportButton = new GeonkickButton(this);
-        exportButton->setCheckable(true);
-        exportButton->setPosition((width() - 2 * 90 - 45) / 2, height() - 50);
-        exportButton->setUnpressedImage(RkImage(90, 30, RK_IMAGE_RC(export_button)));
-        RK_ACT_BIND(exportButton, toggled, RK_ACT_ARGS(bool b), this, exportKick());
-
-        cancelButton = new GeonkickButton(this);
-        cancelButton->setCheckable(true);
-        cancelButton->setPosition((width() - 2 * 90 - 45) / 2 + 90 + 45, height() - 50);
-        cancelButton->setUnpressedImage(RkImage(90, 30, RK_IMAGE_RC(export_cancel)));
-        RK_ACT_BIND(cancelButton, toggled, RK_ACT_ARGS(bool b), this, close());
         show();
 }
 
@@ -154,54 +102,65 @@ ExportWidget::~ExportWidget()
 
 void ExportWidget::createFormatButtons()
 {
-        int y = 19;
+        auto topContainer = new RkContainer(this);
+        topContainer->setSpacing(4);
+        topContainer->setSize({width(), 16});
+        topContainer->setPosition({10, 10});
         flac16Button = new GeonkickButton(this);
         flac16Button->setPressed(ExportFormat::Flac16 == selectedFormat);
         flac16Button->setFixedSize(63, 16);
         flac16Button->setUnpressedImage(RkImage(flac16Button->size(), RK_IMAGE_RC(flac16_radio_button)));
         flac16Button->setPressedImage(RkImage(flac16Button->size(), RK_IMAGE_RC(flac16_radio_button_active)));
-        flac16Button->setPosition(34, y);
         RK_ACT_BIND(flac16Button, toggled, RK_ACT_ARGS(bool b), this, setFormat(ExportFormat::Flac16));
+        topContainer->addWidget(flac16Button);
+        topContainer->addSpace(4);
 
         flac24Button = new GeonkickButton(this);
         flac24Button->setPressed(ExportFormat::Flac24 == selectedFormat);
         flac24Button->setFixedSize(65, 16);
         flac24Button->setUnpressedImage(RkImage(flac24Button->size(), RK_IMAGE_RC(flac24_radio_button)));
         flac24Button->setPressedImage(RkImage(flac24Button->size(), RK_IMAGE_RC(flac24_radio_button_active)));
-        flac24Button->setPosition(34 + 79, y);
         RK_ACT_BIND(flac24Button, toggled, RK_ACT_ARGS(bool b), this, setFormat(ExportFormat::Flac24));
+        topContainer->addWidget(flac24Button);
+        topContainer->addSpace(4);
 
         wav16Button = new GeonkickButton(this);
         wav16Button->setPressed(ExportFormat::Wav16 == selectedFormat);
         wav16Button->setFixedSize(62, 16);
         wav16Button->setUnpressedImage(RkImage(wav16Button->size(), RK_IMAGE_RC(wav16_radio_button)));
         wav16Button->setPressedImage(RkImage(wav16Button->size(), RK_IMAGE_RC(wav16_radio_button_active)));
-        wav16Button->setPosition(34 + 2 * 79, y);
         RK_ACT_BIND(wav16Button, toggled, RK_ACT_ARGS(bool b), this, setFormat(ExportFormat::Wav16));
+        topContainer->addWidget(wav16Button);
+        topContainer->addSpace(4);
 
         wav24Button = new GeonkickButton(this);
         wav24Button->setPressed(ExportFormat::Wav24 == selectedFormat);
         wav24Button->setFixedSize(62, 16);
         wav24Button->setUnpressedImage(RkImage(wav24Button->size(), RK_IMAGE_RC(wav24_radio_button)));
         wav24Button->setPressedImage(RkImage(wav24Button->size(), RK_IMAGE_RC(wav24_radio_button_active)));
-        wav24Button->setPosition(34 + 3 * 79, y);
         RK_ACT_BIND(wav24Button, toggled, RK_ACT_ARGS(bool b), this, setFormat(ExportFormat::Wav24));
+        topContainer->addWidget(wav24Button);
+        topContainer->addSpace(4);
 
         wav32Button = new GeonkickButton(this);
         wav32Button->setPressed(ExportFormat::Wav32 == selectedFormat);
         wav32Button->setFixedSize(62, 16);
         wav32Button->setUnpressedImage(RkImage(wav32Button->size(), RK_IMAGE_RC(wav32_radio_button)));
         wav32Button->setPressedImage(RkImage(wav32Button->size(), RK_IMAGE_RC(wav32_radio_button_active)));
-        wav32Button->setPosition(34 + 4 * 79, y);
         RK_ACT_BIND(wav32Button, toggled, RK_ACT_ARGS(bool b), this, setFormat(ExportFormat::Wav32));
+        topContainer->addWidget(wav32Button);
+        topContainer->addSpace(4);
 
         oggButton = new GeonkickButton(this);
         oggButton->setPressed(ExportFormat::Ogg == selectedFormat);
         oggButton->setFixedSize(51, 16);
         oggButton->setUnpressedImage(RkImage(oggButton->size(), RK_IMAGE_RC(ogg_radio_button)));
         oggButton->setPressedImage(RkImage(oggButton->size(), RK_IMAGE_RC(ogg_radio_button_active)));
-        oggButton->setPosition(34 + 5 * 79, y);
         RK_ACT_BIND(oggButton, toggled, RK_ACT_ARGS(bool b), this, setFormat(ExportFormat::Ogg));
+        topContainer->addWidget(oggButton);
+
+        topContainer->addSpace(16);
+        createChannelsButtons(topContainer);
 }
 
 void ExportWidget::setFormat(ExportFormat format)
@@ -216,29 +175,26 @@ void ExportWidget::setFormat(ExportFormat format)
         wav32Button->setPressed(ExportFormat::Wav32 == format);
         oggButton->setPressed(ExportFormat::Ogg == format);
         selectedFormat = format;
-        resetProgressBar();
-        showError();
 }
 
-void ExportWidget::createChannelsButtons()
+void ExportWidget::createChannelsButtons(RkContainer* container)
 {
-        int y = 95;
-        int x = 192;
         monoButton = new GeonkickButton(this);
         monoButton->setPressed(ChannelsType::Mono == channelsType);
         monoButton->setFixedSize(51, 16);
-        monoButton->setPosition(x , y);
         monoButton->setUnpressedImage(RkImage(monoButton->size(), RK_IMAGE_RC(mono_radio_button)));
         monoButton->setPressedImage(RkImage(monoButton->size(), RK_IMAGE_RC(mono_radio_button_active)));
         RK_ACT_BIND(monoButton, toggled, RK_ACT_ARGS(bool b), this, setChannels(ChannelsType::Mono));
+        container->addWidget(monoButton);
+        container->addSpace(4);
 
         stereoButton = new GeonkickButton(this);
         stereoButton->setPressed(ChannelsType::Stereo == channelsType);
         stereoButton->setFixedSize(56, 16);
-        stereoButton->setPosition(x + 66, y);
         stereoButton->setUnpressedImage(RkImage(stereoButton->size(), RK_IMAGE_RC(stereo_radio_button)));
         stereoButton->setPressedImage(RkImage(stereoButton->size(), RK_IMAGE_RC(stereo_radio_button_active)));
         RK_ACT_BIND(stereoButton, toggled, RK_ACT_ARGS(bool b), this, setChannels(ChannelsType::Stereo));
+        container->addWidget(stereoButton);
 }
 
 void ExportWidget::setChannels(ChannelsType channels)
@@ -255,61 +211,18 @@ void ExportWidget::setChannels(ChannelsType channels)
         }
 
         channelsType = channels;
-        resetProgressBar();
-        showError();
 }
 
-void ExportWidget::browse()
+void ExportWidget::exportInstrument(const std::string &filePath)
 {
-        locationEdit->setFocus(false);
-        fileNameEdit->setFocus(false);
-        auto fileDialog = new FileDialog(this, FileDialog::Type::Open,
-                                         "Select Path - " + std::string(GEONKICK_NAME));
-        fileDialog->setPosition(20, 20);
-        fileDialog->setHomeDirectory(geonkickApi->getSettings("GEONKICK_CONFIG/HOME_PATH"));
-        fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("ExportDialog/Location").string());
-        RK_ACT_BIND(fileDialog, selectedFile,
-                    RK_ACT_ARGS(const std::string &file), this,
-                    setLocation(fileDialog->currentDirectory()));
-}
-
-void ExportWidget::setLocation(const std::string &location)
-{
-        locationEdit->setText(location);
-        geonkickApi->setCurrentWorkingPath("ExportDialog/Location", std::filesystem::path(location));
-}
-
-bool ExportWidget::validateInput()
-{
-        if (locationEdit->text().empty()) {
-                showError("File location is empty");
-                return false;
-        }
-
-        if (fileNameEdit->text().empty()) {
-                showError("File name is empty");
-                return false;
-        }
-
-        return true;
-}
-
-void ExportWidget::exportKick()
-{
-        resetProgressBar();
-        if (!validateInput())
-                return;
-
         SF_INFO sndinfo;
         memset(&sndinfo, 0, sizeof(sndinfo));
         sndinfo.samplerate = geonkickApi->getSampleRate();
-        if (sndinfo.samplerate == 0) {
-                showError("Error on exporting kick");
+        if (sndinfo.samplerate == 0)
                 return;
-        }
 
-        sndinfo.channels   = channelsType == ChannelsType::Mono ? 1 : 2;
-        sndinfo.format     = exportFormat();
+        sndinfo.channels = channelsType == ChannelsType::Mono ? 1 : 2;
+        sndinfo.format   = exportFormat();
 
         auto tempBuffer = geonkickApi->getKickBuffer();
         sndinfo.frames = tempBuffer.size();
@@ -325,46 +238,30 @@ void ExportWidget::exportKick()
                 kickBuffer = std::move(tempBuffer);
         }
 
-        if (kickBuffer.empty() || !sf_format_check(&sndinfo)) {
-                showError("Error: error on exporting kick");
+        auto exportedFilePath = getExportedFilePath(filePath);
+        if (exportedFilePath.empty())
                 return;
-        }
 
-        auto filePath = getFilePath();
-        if (filePath.empty()) {
-                showError("Wrong file name format");
+        auto sndFile = sf_open(exportedFilePath.string().c_str(), SFM_WRITE, &sndinfo);
+        if (!sndFile)
                 return;
-        }
 
-        SNDFILE *sndFile = sf_open(filePath.c_str(), SFM_WRITE, &sndinfo);
-        if (!sndFile) {
-                showError("Error on exporting kick1");
-                return;
-        }
-
-        size_t n;
 #ifdef GEONKICK_DOUBLE_PRECISION
-        n = sf_write_double(sndFile, kickBuffer.data(), kickBuffer.size());
+        sf_write_double(sndFile, kickBuffer.data(), kickBuffer.size());
 #else
-        n = sf_write_float(sndFile, kickBuffer.data(), kickBuffer.size());
+        sf_write_float(sndFile, kickBuffer.data(), kickBuffer.size());
 #endif
-        if (n != kickBuffer.size())
-                showError("Error on exporting");
-        else
-                progressBar->setValue(100);
 
         sf_close(sndFile);
-        close();
 
-        if (!fileNameEdit->text().empty())
-                geonkickApi->setSettings("ExportDialog/FileName", fileNameEdit->text());
+        if (!exportedFilePath.empty()
+            && exportedFilePath.has_parent_path()) {
+                geonkickApi->setCurrentWorkingPath("ExportDialog/Location",
+                                                   exportedFilePath.parent_path().string());
+        }
         geonkickApi->setSettings("ExportDialog/Format", std::to_string(static_cast<int>(selectedFormat)));
         geonkickApi->setSettings("ExportDialog/Channel", std::to_string(static_cast<int>(channelsType)));
-}
-
-void ExportWidget::resetProgressBar()
-{
-        progressBar->reset();
+        close();
 }
 
 int ExportWidget::exportFormat()
@@ -388,19 +285,19 @@ int ExportWidget::exportFormat()
         }
 }
 
-std::string ExportWidget::getFilePath()
+std::filesystem::path
+ExportWidget::getExportedFilePath(const std::filesystem::path &file)
 {
-        auto path = std::filesystem::path(fileNameEdit->text());
-        std::string ext = path.extension().string();
+        auto filePath = file;
+        std::string ext = file.extension().string();
         if (ext == ".wav" || ext == ".WAV"
             || ext == ".flac" || ext == ".FLAC"
             || ext == ".ogg" || ext == ".OGG") {
-                path.replace_extension("." + fileSuffix());
+                filePath.replace_extension("." + fileSuffix());
         } else {
-                path = std::filesystem::path(path.string() + "." + fileSuffix());
+                filePath = std::filesystem::path(filePath.string() + "." + fileSuffix());
         }
-
-        return (std::filesystem::path(locationEdit->text()) / path).string();
+        return filePath;
 }
 
 std::string ExportWidget::fileSuffix()
@@ -420,16 +317,3 @@ std::string ExportWidget::fileSuffix()
                 return std::string();
         }
 }
-
-void ExportWidget::showError(const std::string &error)
-{
-        if (error.empty()) {
-                errorLabel->hide();
-                progressBar->setProgressColor({52, 116, 209});
-        } else {
-                errorLabel->setText(error);
-                errorLabel->show();
-                progressBar->setProgressColor({200, 0, 0});
-                progressBar->setValue(100);
-        }
-}
