@@ -60,12 +60,10 @@ FilesView::FilesView(GeonkickWidget *parent)
         , scrollBar{nullptr}
         , isScrollBarVisible{false}
 {
-        setFixedSize(parent->width() - 110, parent->height() - 85);
+        setFixedSize(parent->width() - 108, parent->height() - 85);
         visibleLines = height() / (lineHeight + lineSacing);
-        setPosition(104, 40);
+        setPosition(106, 40);
         setBackgroundColor(50, 50, 50);
-        setBorderColor(40, 40, 40);
-        setBorderWidth(1);
         createScrollBar();
         showScrollBar(false);
         show();
@@ -381,8 +379,9 @@ void FilesView::setFilters(const std::vector<std::string> &filters)
 
 FileDialog::FileDialog(GeonkickWidget *parent,
                        FileDialog::Type type,
+                       Rk::WidgetFlags flags,
                        const std::string& title)
-        : GeonkickWidget(parent, type == FileDialog::Type::Browse ? Rk::WindowFlags::Widget : Rk::WindowFlags::Popup)
+        : GeonkickWidget(parent, flags)
         , dialogType{type}
         , filesView{nullptr}
         , status{AcceptStatus::Cancel}
@@ -391,10 +390,34 @@ FileDialog::FileDialog(GeonkickWidget *parent,
         , bookmarkDirectoryButton{nullptr}
 {
         setTitle(title);
-        setFixedSize(600, 400);
-        setBorderWidth(1);
-        setBorderColor(40, 40, 40);
+        setFixedSize(600, 370);
+        createUi();
+        show();
+}
 
+FileDialog::FileDialog(GeonkickWidget *parent,
+                       FileDialog::Type type,
+                       const std::string& title)
+        : GeonkickWidget(parent, type == FileDialog::Type::Browse ? Rk::WidgetFlags::Widget : Rk::WidgetFlags::Popup)
+        , dialogType{type}
+        , filesView{nullptr}
+        , status{AcceptStatus::Cancel}
+        , shortcutDirectoriesModel{new PathListModel(this)}
+        , shortcutDirectoriesView{new RkList(this, shortcutDirectoriesModel)}
+        , bookmarkDirectoryButton{nullptr}
+{
+        setTitle(title);
+        setFixedSize(600, 370);
+        createUi();
+        show();
+}
+
+void FileDialog::createUi()
+{
+        if (widgetFlags() == Rk::WidgetFlags::Popup) {
+                setBorderWidth(2);
+                setBorderColor(80, 80, 80);
+        }
         GeonkickConfig cfg;
         for(const auto &path: cfg.getBookmarkedPaths())
                 shortcutDirectoriesModel->addPath(path);
@@ -419,15 +442,13 @@ FileDialog::FileDialog(GeonkickWidget *parent,
                     filesView,
                     setCurrentPath(std::get<std::string>(item.data(static_cast<int>(PathListModel::PathListDataType::Path)))));
 
-        shortcutDirectoriesView->setBorderColor(40, 40, 40);
-        shortcutDirectoriesView->setBorderWidth(1);
         shortcutDirectoriesView->setBackgroundColor({50, 50, 50});
-        shortcutDirectoriesView->setPosition(0, filesView->y());
+        shortcutDirectoriesView->setPosition(2, filesView->y());
         shortcutDirectoriesView->setSize(100 ,filesView->height());
         shortcutDirectoriesView->show();
 
         createBookmarkDirectoryControls(topContainer);
-        //        createNewDirectoryControls(topContainer);
+        createNewDirectoryControls(topContainer);
 
         auto buttomContainer = new RkContainer(this);
         buttomContainer->setSize({mainContainer->width(), 30});
@@ -462,8 +483,6 @@ FileDialog::FileDialog(GeonkickWidget *parent,
                 RK_ACT_BIND(fileNameEdit, enterPressed, RK_ACT_ARGS(), this, onAccept());
                 buttomContainer->addWidget(fileNameEdit);
         }
-
-        show();
 }
 
 void FileDialog::createBookmarkDirectoryControls(RkContainer *container)
@@ -511,11 +530,12 @@ void FileDialog::createNewDirectoryControls(RkContainer *container)
         auto editDirectoryName = new RkLineEdit(this);
         editDirectoryName->setSize(80, 16);
         container->addWidget(editDirectoryName);
+        editDirectoryName->hide();
 
         RK_ACT_BINDL(createDirectoryButton,
                      pressed,
                      RK_ACT_ARGS(),
-                     [=,this](){
+                     [=, this](){
                              createDirectoryButton->hide();
                              editDirectoryName->show();
                              container->update();
@@ -525,7 +545,7 @@ void FileDialog::createNewDirectoryControls(RkContainer *container)
         RK_ACT_BINDL(editDirectoryName,
                      escapePressed,
                      RK_ACT_ARGS(),
-                     [=,this](){
+                     [=, this](){
                              editDirectoryName->hide();
                              createDirectoryButton->show();
                              editDirectoryName->setText("");
@@ -535,7 +555,7 @@ void FileDialog::createNewDirectoryControls(RkContainer *container)
         RK_ACT_BINDL(editDirectoryName,
                      enterPressed,
                      RK_ACT_ARGS(),
-                     [=,this](){
+                     [=, this](){
                              editDirectoryName->hide();
                              createDirectoryButton->show();
                              createDirectory(editDirectoryName->text());
@@ -546,8 +566,8 @@ void FileDialog::createNewDirectoryControls(RkContainer *container)
         RK_ACT_BINDL(editDirectoryName,
                      editingFinished,
                      RK_ACT_ARGS(),
-                     [=,this](){
-                             editDirectoryName->close();
+                     [=, this](){
+                             editDirectoryName->hide();
                              createDirectoryButton->show();
                              editDirectoryName->setText("");
                              container->update();
@@ -591,6 +611,7 @@ void FileDialog::onAccept()
 void FileDialog::onCancel()
 {
         status = AcceptStatus::Cancel;
+        action rejected();
         close();
 }
 
