@@ -34,7 +34,7 @@ RK_DECLARE_IMAGE_RC(hboxbk_filter);
 RK_DECLARE_IMAGE_RC(filter_enabled);
 RK_DECLARE_IMAGE_RC(filter_enabled_hover);
 RK_DECLARE_IMAGE_RC(filter_disabled);
-RK_DECLARE_IMAGE_RC(knob_bk_50x50);
+RK_DECLARE_IMAGE_RC(knob_bk_60x60);
 RK_DECLARE_IMAGE_RC(knob_50x50);
 RK_DECLARE_IMAGE_RC(filter_type_lp);
 RK_DECLARE_IMAGE_RC(filter_type_hp);
@@ -83,8 +83,8 @@ Filter::Filter(GeonkickWidget *parent, Envelope::Category category)
 
         cutOffKnob = new Knob(this);
         cutOffKnob->setRangeType(Knob::RangeType::Logarithmic);
-        cutOffKnob->setPosition((224 / 2 - 80) / 2, (125 - 80) / 2);
         cutOffKnob->setFixedSize(80, 78);
+        cutOffKnob->setPosition((224 / 2 - 80) / 2, (125 - 80) / 2 - 5);
         cutOffKnob->setKnobBackgroundImage(RkImage(80, 80, RK_IMAGE_RC(knob_bk_image)));
         cutOffKnob->setKnobImage(RkImage(70, 70, RK_IMAGE_RC(knob)));
         cutOffKnob->setRange(20, 20000);
@@ -95,7 +95,7 @@ Filter::Filter(GeonkickWidget *parent, Envelope::Category category)
                                          && viewState()->getEnvelopeCategory() == envelopeCategory);
         cutoffEnvelopeButton->setFixedSize(63, 21);
         cutoffEnvelopeButton->setPosition(cutOffKnob->x() + cutOffKnob->width() / 2 - cutoffEnvelopeButton->width() / 2,
-                                          cutOffKnob->y() + cutOffKnob->height());
+                                          cutOffKnob->y() + cutOffKnob->height() + 2);
         cutoffEnvelopeButton->setImage(RkImage(cutoffEnvelopeButton->size(), RK_IMAGE_RC(fl_cutoff_button_off)),
                                        RkButton::State::Unpressed);
         cutoffEnvelopeButton->setImage(RkImage(cutoffEnvelopeButton->size(), RK_IMAGE_RC(fl_cutoff_button_on)),
@@ -119,7 +119,7 @@ Filter::Filter(GeonkickWidget *parent, Envelope::Category category)
         int h = 60;
         resonanceKnob->setPosition(224 / 2  + (224 / 2 - w) / 2, (125 - h) / 4 - 2);
         resonanceKnob->setFixedSize(w, h);
-        resonanceKnob->setKnobBackgroundImage(RkImage(w, h, RK_IMAGE_RC(knob_bk_50x50)));
+        resonanceKnob->setKnobBackgroundImage(RkImage(w, h, RK_IMAGE_RC(knob_bk_60x60)));
         resonanceKnob->setKnobImage(RkImage(50, 50, RK_IMAGE_RC(knob_50x50)));
         resonanceKnob->setRange(0.01, 10);
         RK_ACT_BIND(resonanceKnob, valueUpdated, RK_ACT_ARGS(double val), this, resonanceChanged(val));
@@ -203,8 +203,9 @@ bool Filter::isEnabled() const
         return filterCheckbox->isPressed();
 }
 
-void Filter::setCutOff(double val)
+void Filter::setCutOff(double val, double defaultVal)
 {
+        cutOffKnob->setDefaultValue(defaultVal);
         cutOffKnob->setCurrentValue(val);
 }
 
@@ -213,8 +214,9 @@ double Filter::cutOff() const
         return cutOffKnob->getValue();
 }
 
-void Filter::setResonance(double val)
+void Filter::setResonance(double val, double defaultVal)
 {
+        resonanceKnob->setDefaultValue(defaultVal);
         resonanceKnob->setCurrentValue(val);
 }
 
