@@ -33,6 +33,7 @@
 #include "GeonkickConfig.h"
 
 #include <RkEventQueue.h>
+#include <RkAction.h>
 
 #include <sndfile.h>
 
@@ -83,8 +84,8 @@ void GeonkickApi::setEventQueue(RkEventQueue *queue)
 {
         std::lock_guard<std::mutex> lock(apiMutex);
         eventQueue = queue;
-        if (eventQueue)
-                eventQueue->setScaleFactor(getScaleFactor());
+        //        if (eventQueue)
+        //                eventQueue->setScaleFactor(getScaleFactor());
 }
 
 bool GeonkickApi::initDSP()
@@ -143,6 +144,11 @@ size_t GeonkickApi::numberOfMidiChannels()
         return geonkick_midi_channels_number();
 }
 
+size_t GeonkickApi::numberOfLayers()
+{
+        return geonkick_layers_number();
+}
+
 std::unique_ptr<KitState> GeonkickApi::getDefaultKitState()
 {
         return std::make_unique<KitState>();
@@ -173,13 +179,6 @@ std::unique_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
 	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor, envelope);
 	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive, envelope);
         state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume, envelope);
-        state->enableCompressor(false);
-        state->setCompressorAttack(0.01);
-        state->setCompressorRelease(0.01);
-        state->setCompressorThreshold(0);
-        state->setCompressorRatio(1);
-        state->setCompressorKnee(0);
-        state->setCompressorMakeup(1);
         state->enableDistortion(false);
         state->setDistortionVolume(0.1);
         state->setDistortionInLimiter(1.0);
@@ -188,49 +187,36 @@ std::unique_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
         std::vector<GeonkickApi::OscillatorType> oscillators = {
                 GeonkickApi::OscillatorType::Oscillator1,
                 GeonkickApi::OscillatorType::Oscillator2,
-                GeonkickApi::OscillatorType::Noise
+                GeonkickApi::OscillatorType::Oscillator3
         };
 
-        std::vector<GeonkickApi::Layer> layers = {
-                GeonkickApi::Layer::Layer1,
-                GeonkickApi::Layer::Layer2,
-                GeonkickApi::Layer::Layer3
-        };
-
-        for (auto layer: layers) {
+        auto nLayers = numberOfLayers();
+        for (size_t l = 0; l < nLayers; l++) {
+                auto layer = static_cast<Layer>(l);
                 state->setLayerEnabled(layer, layer == Layer::Layer1);
                 state->setLayerAmplitude(layer, 1.0);
                 for (auto const &osc: oscillators) {
                         int index = static_cast<int>(osc) + GKICK_OSC_GROUP_SIZE * static_cast<int>(layer);
                         state->setOscillatorEnabled(index, osc == GeonkickApi::OscillatorType::Oscillator1);
-                        if (osc == GeonkickApi::OscillatorType::Noise) {
-                                state->setOscillatorFunction(index, GeonkickApi::FunctionType::NoiseWhite);
-                                state->setOscillatorSeed(index, 100);
-                        } else {
-                                state->setOscillatorFunction(index, GeonkickApi::FunctionType::Sine);
-                                state->setOscillatorPhase(index, 0);
-                        }
-
+                        state->setOscillatorFunction(index, GeonkickApi::FunctionType::Sine);
+                        state->setOscillatorPhase(index, 0);
                         state->setOscillatorAmplitue(index, 0.26);
                         state->setOscillatorFrequency(index, 800);
-                        state->setOscillatorPitchShift(index, 0);
+                        state->setOscillatorPitchShift(index, 12);
                         state->setOscillatorFilterEnabled(index, false);
                         state->setOscillatorFilterType(index, GeonkickApi::FilterType::LowPass);
                         state->setOscillatorFilterCutOffFreq(index, 800);
                         state->setOscillatorFilterFactor(index, 10);
                         state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Amplitude);
-                        if (osc != GeonkickApi::OscillatorType::Noise) {
-				state->setOscillatorEnvelopeApplyType(index,
-								      GeonkickApi::EnvelopeType::Frequency,
-								      GeonkickApi::EnvelopeApplyType::Logarithmic);
-                                state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Frequency);
-                                state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::PitchShift);
-                                std::vector<RkRealPoint> env = envelope;
-                                env[0].setY(0.5);
-                                env[1].setY(0.5);
-                                state->setOscillatorEnvelopePoints(index, env, GeonkickApi::EnvelopeType::PitchShift);
-
-                        }
+                        state->setOscillatorEnvelopeApplyType(index,
+                                                              GeonkickApi::EnvelopeType::Frequency,
+                                                              GeonkickApi::EnvelopeApplyType::Logarithmic);
+                        state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Frequency);
+                        state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::PitchShift);
+                        std::vector<RkRealPoint> env = envelope;
+                        env[0].setY(0.5);
+                        env[1].setY(0.5);
+                        state->setOscillatorEnvelopePoints(index, env, GeonkickApi::EnvelopeType::PitchShift);
                         state->setOscillatorEnvelopePoints(index,
 							   envelope,
 							   GeonkickApi::EnvelopeType::FilterCutOff);
@@ -240,7 +226,6 @@ std::unique_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
 			state->setOscillatorEnvelopeApplyType(index,
 							      GeonkickApi::EnvelopeType::FilterCutOff,
 							      GeonkickApi::EnvelopeApplyType::Logarithmic);
-                        state->setOscillatorPitchShift(index, 12);
                 }
         }
 
@@ -260,9 +245,11 @@ void GeonkickApi::setPercussionState(const std::unique_ptr<PercussionState> &sta
         setPercussionPlayingKey(state->getId(), state->getPlayingKey());
         setPercussionChannel(state->getId(), state->getChannel());
         setPercussionMidiChannel(state->getId(), state->getMidiChannel());
+        enableNoteOff(state->getId(), state->isNoteOffEnabled());
         mutePercussion(state->getId(), state->isMuted());
         soloPercussion(state->getId(), state->isSolo());
-        for (auto i = 0; i < 3; i++) {
+        auto nLayers = numberOfLayers();
+        for (size_t i = 0; i < nLayers; i++) {
                 enbaleLayer(static_cast<Layer>(i), state->isLayerEnabled(static_cast<Layer>(i)));
                 setLayerAmplitude(static_cast<Layer>(i), state->getLayerAmplitude(static_cast<Layer>(i)));
         }
@@ -287,18 +274,11 @@ void GeonkickApi::setPercussionState(const std::unique_ptr<PercussionState> &sta
         setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume,
                               state->getKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume));
 
-        for (auto i = 0; i < 3; i++) {
+        for (size_t i = 0; i < nLayers; i++) {
                 setOscillatorState(static_cast<Layer>(i), OscillatorType::Oscillator1, state);
                 setOscillatorState(static_cast<Layer>(i), OscillatorType::Oscillator2, state);
-                setOscillatorState(static_cast<Layer>(i), OscillatorType::Noise, state);
+                setOscillatorState(static_cast<Layer>(i), OscillatorType::Oscillator3, state);
         }
-        enableCompressor(state->isCompressorEnabled());
-        setCompressorAttack(state->getCompressorAttack());
-        setCompressorRelease(state->getCompressorRelease());
-        setCompressorThreshold(state->getCompressorThreshold());
-        setCompressorRatio(state->getCompressorRatio());
-        setCompressorKnee(state->getCompressorKnee());
-        setCompressorMakeup(state->getCompressorMakeup());
         enableDistortion(state->isDistortionEnabled());
         setDistortionInLimiter(state->getDistortionInLimiter());
         setDistortionVolume(state->getDistortionVolume());
@@ -342,9 +322,11 @@ std::unique_ptr<PercussionState> GeonkickApi::getPercussionState() const
         state->setPlayingKey(getPercussionPlayingKey(state->getId()));
         state->setChannel(getPercussionChannel(state->getId()));
         state->setMidiChannel(getPercussionMidiChannel(state->getId()));
+        state->setNoteOffEnabled(isNoteOffEnabled(state->getId()));
         state->setMute(isPercussionMuted(state->getId()));
         state->setSolo(isPercussionSolo(state->getId()));
-        for (int i = 0; i < 3; i++) {
+        auto nLayers = numberOfLayers();
+        for (size_t i = 0; i < nLayers; i++) {
                 state->setLayerEnabled(static_cast<Layer>(i), isLayerEnabled(static_cast<Layer>(i)));
                 state->setLayerAmplitude(static_cast<Layer>(i), getLayerAmplitude(static_cast<Layer>(i)));
         }
@@ -368,18 +350,11 @@ std::unique_ptr<PercussionState> GeonkickApi::getPercussionState() const
                                      getKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume));
 
 
-        for (int i = 0; i < 3; i++) {
+        for (size_t i = 0; i < nLayers; i++) {
                 getOscillatorState(static_cast<Layer>(i), OscillatorType::Oscillator1, state);
                 getOscillatorState(static_cast<Layer>(i), OscillatorType::Oscillator2, state);
-                getOscillatorState(static_cast<Layer>(i), OscillatorType::Noise, state);
+                getOscillatorState(static_cast<Layer>(i), OscillatorType::Oscillator3, state);
         }
-        state->enableCompressor(isCompressorEnabled());
-        state->setCompressorAttack(getCompressorAttack());
-        state->setCompressorRelease(getCompressorRelease());
-        state->setCompressorThreshold(getCompressorThreshold());
-        state->setCompressorRatio(getCompressorRatio());
-        state->setCompressorKnee(getCompressorKnee());
-        state->setCompressorMakeup(getCompressorMakeup());
         state->enableDistortion(isDistortionEnabled());
         state->setDistortionInLimiter(getDistortionInLimiter());
         state->setDistortionVolume(getDistortionVolume());
@@ -399,28 +374,27 @@ void GeonkickApi::getOscillatorState(GeonkickApi::Layer layer,
         state->setOscillatorEnabled(index, isOscillatorEnabled(index));
         state->setOscillatorFunction(index, oscillatorFunction(index));
         state->setOscillatorSample(index, getOscillatorSample(index));
-        if (osc != OscillatorType::Noise)
-                state->setOscillatorPhase(index, oscillatorPhase(index));
-        if (osc == OscillatorType::Noise)
-                state->setOscillatorSeed(index, oscillatorSeed(index));
+        state->setOscillatorPhase(index, oscillatorPhase(index));
+        state->setOscillatorSeed(index, oscillatorSeed(index));
         state->setOscillatorAmplitue(index, oscillatorAmplitude(index));
         state->setOscillatorFrequency(index, oscillatorFrequency(index));
         state->setOscillatorPitchShift(index, oscillatorPitchShift(index));
+        state->setOscillatorNoiseDensity(index, oscillatorNoiseDensity(index));
         state->setOscillatorFilterEnabled(index, isOscillatorFilterEnabled(index));
         state->setOscillatorFilterType(index, getOscillatorFilterType(index));
         state->setOscillatorFilterCutOffFreq(index, getOscillatorFilterCutOffFreq(index));
         state->setOscillatorFilterFactor(index, getOscillatorFilterFactor(index));
         auto points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Amplitude);
         state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::Amplitude);
-        if (osc != OscillatorType::Noise) {
-		auto applyType = getOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::Frequency);
-	        state->setOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::Frequency, applyType);
-		points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Frequency);
-                state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::Frequency);
-                points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::PitchShift);
-                state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::PitchShift);
-        }
-        auto applyType = getOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::FilterCutOff);
+        auto applyType = getOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::Frequency);
+        state->setOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::Frequency, applyType);
+        points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Frequency);
+        state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::Frequency);
+        points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::PitchShift);
+        state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::PitchShift);
+        points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::NoiseDensity);
+        state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::NoiseDensity);
+        applyType = getOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::FilterCutOff);
         state->setOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::FilterCutOff, applyType);
         points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::FilterCutOff);
         state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::FilterCutOff);
@@ -441,32 +415,30 @@ void GeonkickApi::setOscillatorState(GeonkickApi::Layer layer,
         enableOscillator(osc, state->isOscillatorEnabled(osc));
         setOscillatorFunction(osc, state->oscillatorFunction(osc));
         setOscillatorSample(state->getOscillatorSample(osc), osc);
-        if (oscillator != OscillatorType::Noise)
-                setOscillatorPhase(osc, state->oscillatorPhase(osc));
-        if (oscillator == OscillatorType::Noise)
-                setOscillatorSeed(osc, state->oscillatorSeed(osc));
+        setOscillatorPhase(osc, state->oscillatorPhase(osc));
+        setOscillatorSeed(osc, state->oscillatorSeed(osc));
         setOscillatorAmplitude(osc, state->oscillatorAmplitue(osc));
-        if (oscillator != OscillatorType::Noise) {
-                setOscillatorFrequency(osc, state->oscillatorFrequency(osc));
-                setOscillatorPitchShift(osc, state->oscillatorPitchShift(osc));
-        }
+        setOscillatorFrequency(osc, state->oscillatorFrequency(osc));
+        setOscillatorPitchShift(osc, state->oscillatorPitchShift(osc));
+        setOscillatorNoiseDensity(osc, state->oscillatorNoiseDensity(osc));
         enableOscillatorFilter(osc, state->isOscillatorFilterEnabled(osc));
         setOscillatorFilterType(osc, state->oscillatorFilterType(osc));
         setOscillatorFilterCutOffFreq(osc, state->oscillatorFilterCutOffFreq(osc));
         setOscillatorFilterFactor(osc, state->oscillatorFilterFactor(osc));
         setOscillatorEvelopePoints(osc, EnvelopeType::Amplitude,
                                    state->oscillatorEnvelopePoints(osc, EnvelopeType::Amplitude));
-        if (oscillator != OscillatorType::Noise) {
-		setOscillatorEnvelopeApplyType(osc, EnvelopeType::Frequency,
-					       state->getOscillatorEnvelopeApplyType(osc,
-										     EnvelopeType::Frequency));
-                setOscillatorEvelopePoints(osc, EnvelopeType::Frequency,
-                                           state->oscillatorEnvelopePoints(osc,
-                                           EnvelopeType::Frequency));
-                setOscillatorEvelopePoints(osc, EnvelopeType::PitchShift,
-                                           state->oscillatorEnvelopePoints(osc,
-                                           EnvelopeType::PitchShift));
-        }
+        setOscillatorEnvelopeApplyType(osc, EnvelopeType::Frequency,
+                                       state->getOscillatorEnvelopeApplyType(osc,
+                                                                             EnvelopeType::Frequency));
+        setOscillatorEvelopePoints(osc, EnvelopeType::Frequency,
+                                   state->oscillatorEnvelopePoints(osc,
+                                                                   EnvelopeType::Frequency));
+        setOscillatorEvelopePoints(osc, EnvelopeType::PitchShift,
+                                   state->oscillatorEnvelopePoints(osc,
+                                                                   EnvelopeType::PitchShift));
+        setOscillatorEvelopePoints(osc, EnvelopeType::NoiseDensity,
+                                   state->oscillatorEnvelopePoints(osc,
+                                                                   EnvelopeType::NoiseDensity));
 	setOscillatorEnvelopeApplyType(osc, EnvelopeType::FilterCutOff,
 				       state->getOscillatorEnvelopeApplyType(osc,
 									     EnvelopeType::FilterCutOff));
@@ -488,8 +460,8 @@ std::unique_ptr<KitState> GeonkickApi::getKitState() const
         for (const auto &id : ordredPercussionIds()) {
                 auto state = getPercussionState(id);
                 state->setId(i);
+		GEONKICK_LOG_DEBUG("PER: " << state->getName() << ": id = " << state->getId());
                 kit->addPercussion(std::move(state));
-                GEONKICK_LOG_DEBUG("PER: " << state->getName() << ": id = " << state->getId());
                 i++;
         }
         return kit;
@@ -903,6 +875,24 @@ double GeonkickApi::oscillatorPitchShift(int oscillatorIndex) const
 	return semitones;
 }
 
+bool GeonkickApi::setOscillatorNoiseDensity(int oscillatorIndex, double density)
+{
+	return geonkick_set_osc_noise_density(geonkickApi,
+                                              getOscIndex(oscillatorIndex),
+                                              density) == GEONKICK_OK;
+}
+
+double GeonkickApi::oscillatorNoiseDensity(int oscillatorIndex) const
+{
+	gkick_real density = 0;
+	if (geonkick_get_osc_noise_density(geonkickApi,
+                                           getOscIndex(oscillatorIndex),
+                                           &density) != GEONKICK_OK) {
+                return 0;
+        }
+	return density;
+}
+
 bool GeonkickApi::isOscillatorEnabled(int oscillatorIndex) const
 {
         int enabled = 0;
@@ -1095,90 +1085,6 @@ void GeonkickApi::process(float** out, size_t offset, size_t size)
         geonkick_audio_process(geonkickApi, out, offset, size);
 }
 
-void GeonkickApi::enableCompressor(bool enable)
-{
-        geonkick_compressor_enable(geonkickApi, enable);
-}
-
-bool GeonkickApi::isCompressorEnabled() const
-{
-        int enabled = false;
-        geonkick_compressor_is_enabled(geonkickApi, &enabled);
-        return enabled;
-}
-
-void GeonkickApi::setCompressorAttack(double attack)
-{
-        geonkick_compressor_set_attack(geonkickApi, attack);
-}
-
-void GeonkickApi::setCompressorRelease(double release)
-{
-        geonkick_compressor_set_release(geonkickApi, release);
-}
-
-void GeonkickApi::setCompressorThreshold(double threshold)
-{
-        geonkick_compressor_set_threshold(geonkickApi, threshold);
-}
-
-void GeonkickApi::setCompressorRatio(double ratio)
-{
-        geonkick_compressor_set_ratio(geonkickApi, ratio);
-}
-
-void GeonkickApi::setCompressorKnee(double knee)
-{
-        geonkick_compressor_set_knee(geonkickApi, knee);
-}
-
-void GeonkickApi::setCompressorMakeup(double makeup)
-{
-        geonkick_compressor_set_makeup(geonkickApi, makeup);
-}
-
-double GeonkickApi::getCompressorAttack() const
-{
-        gkick_real val = 0;
-        geonkick_compressor_get_attack(geonkickApi, &val);
-        return val;
-}
-
-double GeonkickApi::getCompressorRelease() const
-{
-        gkick_real val = 0;
-        geonkick_compressor_get_release(geonkickApi, &val);
-        return val;
-}
-
-double GeonkickApi::getCompressorThreshold() const
-{
-        gkick_real val = 0;
-        geonkick_compressor_get_threshold(geonkickApi, &val);
-        return val;
-}
-
-double GeonkickApi::getCompressorRatio() const
-{
-        gkick_real val = 0;
-        geonkick_compressor_get_ratio(geonkickApi, &val);
-        return val;
-}
-
-double GeonkickApi::getCompressorKnee() const
-{
-        gkick_real val = 0;
-        geonkick_compressor_get_knee(geonkickApi, &val);
-        return val;
-}
-
-double GeonkickApi::getCompressorMakeup() const
-{
-        gkick_real val = 1.0;
-        geonkick_compressor_get_makeup(geonkickApi, &val);
-        return val;
-}
-
 void GeonkickApi::enableDistortion(bool enable)
 {
         geonkick_distortion_enable(geonkickApi, enable);
@@ -1360,6 +1266,18 @@ bool GeonkickApi::isPercussionSolo(size_t id) const
         return solo;
 }
 
+bool GeonkickApi::enableNoteOff(size_t id, bool b)
+{
+        return geonkick_percussion_enable_note_off(geonkickApi, id, b) == GEONKICK_OK;
+}
+
+bool GeonkickApi::isNoteOffEnabled(size_t id) const
+{
+        bool enabled = false;
+        geonkick_percussion_note_off_enabled(geonkickApi, id, &enabled);
+        return enabled;
+}
+
 int GeonkickApi::getUnusedPercussion() const
 {
         int index;
