@@ -2,7 +2,7 @@
  * File name: geonkick_api.h
  * Project: Geonkick (A percussion synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -54,7 +54,7 @@ class GeonkickApi : public RkObject {
   enum class OscillatorType: int {
          Oscillator1 = 0,
          Oscillator2 = 1,
-         Noise       = 2
+         Oscillator3 = 2
   };
 
   enum class FunctionType: int {
@@ -75,7 +75,8 @@ class GeonkickApi : public RkObject {
 	  FilterQFactor = GEONKICK_FILTER_Q_ENVELOPE,
 	  DistortionDrive = GEONKICK_DISTORTION_DRIVE_ENVELOPE,
           DistortionVolume = GEONKICK_DISTORTION_VOLUME_ENVELOPE,
-          PitchShift = GEONKICK_PITCH_SHIFT_ENVELOPE
+          PitchShift = GEONKICK_PITCH_SHIFT_ENVELOPE,
+          NoiseDensity = GEONKICK_NOISE_DENSITY_ENVELOPE
   };
 
   enum class EnvelopeApplyType: int {
@@ -99,6 +100,7 @@ class GeonkickApi : public RkObject {
   size_t static numberOfInstruments();
   size_t static numberOfChannels();
   size_t static numberOfMidiChannels();
+  size_t static numberOfLayers();
   void setEventQueue(RkEventQueue *queue);
   bool initDSP();
   bool init();
@@ -140,12 +142,16 @@ class GeonkickApi : public RkObject {
                               double frequency);
   bool setOscillatorPitchShift(int oscillatorIndex,
                                double semitones);
+  bool setOscillatorNoiseDensity(int oscillatorIndex,
+                                 double density);
+
 
   void setOscillatorAsFm(int oscillatorIndex, bool b);
   bool isOscillatorAsFm(int oscillatorIndex) const;
   double oscillatorAmplitude(int oscillatorIndex) const;
   double oscillatorFrequency(int oscillatorIndex) const;
   double oscillatorPitchShift(int oscillatorIndex) const;
+  double oscillatorNoiseDensity(int oscillatorIndex) const;
   void addKickEnvelopePoint(EnvelopeType envelope,
                             double x,
                             double y);
@@ -195,13 +201,6 @@ class GeonkickApi : public RkObject {
 
   std::unique_ptr<PercussionState> getPercussionState(size_t id) const;
   std::unique_ptr<PercussionState> getPercussionState() const;
-  bool isCompressorEnabled() const;
-  double getCompressorAttack() const;
-  double getCompressorRelease() const;
-  double getCompressorThreshold() const;
-  double getCompressorRatio() const;
-  double getCompressorKnee() const;
-  double getCompressorMakeup() const;
   bool isDistortionEnabled() const;
   double getDistortionInLimiter() const;
   double getDistortionVolume() const;
@@ -232,13 +231,6 @@ class GeonkickApi : public RkObject {
   void setKickEnvelopeApplyType(EnvelopeType envelope,
 				EnvelopeApplyType applyType);
   void playKick(int id = -1);
-  void enableCompressor(bool enable);
-  void setCompressorAttack(double attack);
-  void setCompressorRelease(double release);
-  void setCompressorThreshold(double threshold);
-  void setCompressorRatio(double ratio);
-  void setCompressorKnee(double knee);
-  void setCompressorMakeup(double makeup);
   void enableDistortion(bool enable);
   void setDistortionVolume(double volume);
   void setDistortionInLimiter(double limit);
@@ -281,6 +273,8 @@ class GeonkickApi : public RkObject {
   bool isPercussionMuted(size_t id) const;
   bool soloPercussion(size_t id, bool b);
   bool isPercussionSolo(size_t id) const;
+  bool enableNoteOff(size_t id, bool b = true);
+  bool isNoteOffEnabled(size_t id) const;
   std::string getPercussionName(int index) const;
   void copyToClipboard();
   void pasteFromClipboard();
