@@ -2,7 +2,7 @@
  * File name: percussion_state.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iuri Nistor 
+ * Copyright (C) 2018 Iuri Nistor
  *
  * This file is part of Geonkick.
  *
@@ -25,6 +25,7 @@
 #include "Base64EncoderDecoder.h"
 
 #include <iomanip>
+#include <algorithm>
 
 PercussionState::PercussionState()
         : appVersion{GEONKICK_VERSION}
@@ -36,6 +37,7 @@ PercussionState::PercussionState()
         , kickEnabled{true}
         , percussionMuted{false}
         , percussionSolo{false}
+        , noteOffEnabled{false}
         , limiterValue{0}
         , kickLength{50}
         , kickAmplitude{0.8}
@@ -44,9 +46,8 @@ PercussionState::PercussionState()
         , kickFilterQFactor{1.0}
         , kickFilterType{GeonkickApi::FilterType::LowPass}
 	, kickCutOffEnvelopeApplyType{GeonkickApi::EnvelopeApplyType::Logarithmic}
-        , compressor{false, 0, 0, 0, 0, 0, 0}
         , distortion{false, 1.0, 1.0, 1.0}
-        , layers{false, false, false}
+        , layers(GeonkickApi::numberOfLayers(), false)
         , layersAmplitude{1.0, 1.0, 1.0}
         , currentLayer{GeonkickApi::Layer::Layer1}
         , tunedOutput{false}
@@ -141,7 +142,7 @@ void PercussionState::setId(size_t id)
         kickId = id;
 }
 
-std::string PercussionState::getName() const
+const std::string& PercussionState::getName() const
 {
         return kickName;
 }
@@ -163,12 +164,20 @@ void PercussionState::setPlayingKey(signed char key)
 
 void PercussionState::setChannel(size_t channel)
 {
-        outputChannel = channel;
+        outputChannel = std::clamp(channel,
+                                   static_cast<size_t>(0),
+                                   GeonkickApi::numberOfChannels() - 2);
 }
 
 void PercussionState::setMidiChannel(signed char channel)
 {
-        outputMidiChannel = channel;
+        if (GeonkickTypes::geonkickAnyMidiChannel == static_cast<int>(channel)) {
+                outputMidiChannel = channel;
+        } else {
+                outputMidiChannel = std::clamp(channel,
+                                               static_cast<decltype(channel)>(0),
+                                               static_cast<decltype(channel)>(GeonkickApi::numberOfMidiChannels() - 1));
+        }
 }
 
 void PercussionState::setMute(bool b)
@@ -201,6 +210,16 @@ signed char PercussionState::getMidiChannel() const
         return outputMidiChannel;
 }
 
+void PercussionState::setNoteOffEnabled(bool b)
+{
+        noteOffEnabled = b;
+}
+
+bool PercussionState::isNoteOffEnabled() const
+{
+        return noteOffEnabled;
+}
+
 bool PercussionState::isEnabled() const
 {
         return kickEnabled;
@@ -220,7 +239,7 @@ void PercussionState::initOscillators()
                 oscillators.insert({static_cast<int>(GeonkickApi::OscillatorType::Oscillator2)
                                 + GKICK_OSC_GROUP_SIZE * i,
                                 OscillatorInfo()});
-                oscillators.insert({static_cast<int>(GeonkickApi::OscillatorType::Noise)
+                oscillators.insert({static_cast<int>(GeonkickApi::OscillatorType::Oscillator3)
                                 + GKICK_OSC_GROUP_SIZE * i,
                                 OscillatorInfo()});
         }
@@ -242,6 +261,8 @@ void PercussionState::parseKickObject(const rapidjson::Value &kick)
                         setChannel(m.value.GetInt());
                 if (m.name == "midiChannel" && m.value.IsInt())
                         setMidiChannel(m.value.GetInt());
+                if (m.name == "noteOffEnabled" && m.value.IsBool())
+                        setNoteOffEnabled(m.value.GetBool());
                 if (m.name == "mute" && m.value.IsBool())
                         setMute(m.value.GetBool());
                 if (m.name == "solo" && m.value.IsBool())
@@ -255,7 +276,7 @@ void PercussionState::parseKickObject(const rapidjson::Value &kick)
                         tuneOutput(m.value.GetBool());
 
                 if (m.name == "layers" && m.value.IsArray()) {
-                        layers = {false, false, false};
+                        layers = std::vector<bool>(GeonkickApi::numberOfLayers(), false);
                         for (const auto &el: m.value.GetArray())
                                 if (el.IsInt())
                                         setLayerEnabled(static_cast<GeonkickApi::Layer>(el.GetInt()), true);
@@ -309,26 +330,6 @@ void PercussionState::parseKickObject(const rapidjson::Value &kick)
 			setKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff, applyType);
                 }
 
-                if (m.name == "compressor" && m.value.IsObject()) {
-                        for (const auto &el: m.value.GetObject()) {
-                                if (el.name == "enabled" && el.value.IsBool())
-                                        enableCompressor(el.value.GetBool());
-                                if (el.name == "attack" && el.value.IsDouble())
-                                        setCompressorAttack(el.value.GetDouble());
-                                if (el.name == "release" && el.value.IsDouble())
-                                        setCompressorRelease(el.value.GetDouble());
-                                if (el.name == "threshold" && el.value.IsDouble())
-                                        setCompressorThreshold(el.value.GetDouble());
-                                if (el.name == "ratio" && el.value.IsDouble())
-                                        setCompressorRatio(el.value.GetDouble());
-                                if (el.name == "knee" && el.value.IsDouble())
-                                        setCompressorKnee(el.value.GetDouble());
-                                if (el.name == "makeup" && el.value.IsDouble())
-                                        setCompressorMakeup(el.value.GetDouble());
-
-                        }
-                }
-
                 if (m.name == "distortion" && m.value.IsObject()) {
                         for (const auto &el: m.value.GetObject()) {
                                 if (el.name == "enabled" && el.value.IsBool())
@@ -379,31 +380,39 @@ void PercussionState::parseOscillatorObject(int index,  const rapidjson::Value &
                         }
                 }
 
-                if (static_cast<GeonkickApi::OscillatorType>(index) != GeonkickApi::OscillatorType::Noise) {
-                        if (m.name == "freq_env" && m.value.IsObject()) {
-				auto applyType = GeonkickApi::EnvelopeApplyType::Linear;
-                                for (const auto &el: m.value.GetObject()) {
-                                        if (el.name == "amplitude" && el.value.IsDouble())
-                                                setOscillatorFrequency(index, el.value.GetDouble());
-					if (el.name == "apply_type")
-						applyType = getApplyTypeFromObj(el.value);
-                                        if (el.name == "points" && el.value.IsArray())
-                                                setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
-                                                                            GeonkickApi::EnvelopeType::Frequency);
-                                }
-			        setOscillatorEnvelopeApplyType(index,
-							       GeonkickApi::EnvelopeType::Frequency,
-							       applyType);
+                if (m.name == "freq_env" && m.value.IsObject()) {
+                        auto applyType = GeonkickApi::EnvelopeApplyType::Linear;
+                        for (const auto &el: m.value.GetObject()) {
+                                if (el.name == "amplitude" && el.value.IsDouble())
+                                        setOscillatorFrequency(index, el.value.GetDouble());
+                                if (el.name == "apply_type")
+                                        applyType = getApplyTypeFromObj(el.value);
+                                if (el.name == "points" && el.value.IsArray())
+                                        setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
+                                                                    GeonkickApi::EnvelopeType::Frequency);
                         }
+                        setOscillatorEnvelopeApplyType(index,
+                                                       GeonkickApi::EnvelopeType::Frequency,
+                                                       applyType);
+                }
 
-                        if (m.name == "pitchshift_env" && m.value.IsObject()) {
-                                for (const auto &el: m.value.GetObject()) {
-                                        if (el.name == "amplitude" && el.value.IsDouble())
-                                                setOscillatorPitchShift(index, el.value.GetDouble());
-                                        if (el.name == "points" && el.value.IsArray())
-                                                setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
-                                                                            GeonkickApi::EnvelopeType::PitchShift);
-                                }
+                if (m.name == "pitchshift_env" && m.value.IsObject()) {
+                        for (const auto &el: m.value.GetObject()) {
+                                if (el.name == "amplitude" && el.value.IsDouble())
+                                        setOscillatorPitchShift(index, el.value.GetDouble());
+                                if (el.name == "points" && el.value.IsArray())
+                                        setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
+                                                                    GeonkickApi::EnvelopeType::PitchShift);
+                        }
+                }
+
+                if (m.name == "noise_density_env" && m.value.IsObject()) {
+                        for (const auto &el: m.value.GetObject()) {
+                                if (el.name == "amplitude" && el.value.IsDouble())
+                                        setOscillatorNoiseDensity(index, el.value.GetDouble());
+                                if (el.name == "points" && el.value.IsArray())
+                                        setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
+                                                                    GeonkickApi::EnvelopeType::NoiseDensity);
                         }
                 }
 
@@ -647,6 +656,12 @@ void PercussionState::setOscillatorPitchShift(int index, double val)
                 oscillator->pitchShift = val;
 }
 
+void PercussionState::setOscillatorNoiseDensity(int index, double val)
+{
+        if (auto oscillator = getOscillator(index); oscillator)
+                oscillator->noiseDensity = val;
+}
+
 void PercussionState::setOscillatorFilterEnabled(int index, bool b)
 {
         auto oscillator = getOscillator(index);
@@ -691,6 +706,9 @@ void PercussionState::setOscillatorEnvelopePoints(int index,
                 case GeonkickApi::EnvelopeType::PitchShift:
                         oscillator->pitchShiftEnvelope = points;
                         break;
+                case GeonkickApi::EnvelopeType::NoiseDensity:
+                        oscillator->noiseDensityEnvelope = points;
+                        break;
                 case GeonkickApi::EnvelopeType::FilterCutOff:
                         oscillator->filterCutOffEnvelope = points;
                         break;
@@ -813,6 +831,13 @@ double PercussionState::oscillatorPitchShift(int index) const
         return 0;
 }
 
+double PercussionState::oscillatorNoiseDensity(int index) const
+{
+        if (auto oscillator = getConstOscillator(index); oscillator)
+                return oscillator->noiseDensity;
+        return 0;
+}
+
 bool PercussionState::isOscillatorFilterEnabled(int index) const
 {
         if (auto oscillator = getConstOscillator(index); oscillator)
@@ -855,6 +880,8 @@ PercussionState::oscillatorEnvelopePoints(int index, GeonkickApi::EnvelopeType t
                         return oscillator->frequencyEnvelope;
                 case GeonkickApi::EnvelopeType::PitchShift:
                         return oscillator->pitchShiftEnvelope;
+                case GeonkickApi::EnvelopeType::NoiseDensity:
+                        return oscillator->noiseDensityEnvelope;
                 case GeonkickApi::EnvelopeType::FilterCutOff:
                         return oscillator->filterCutOffEnvelope;
 		case GeonkickApi::EnvelopeType::FilterQFactor:
@@ -866,76 +893,6 @@ PercussionState::oscillatorEnvelopePoints(int index, GeonkickApi::EnvelopeType t
         return {};
 }
 
-void PercussionState::enableCompressor(bool enable)
-{
-        compressor.enabled = enable;
-}
-
-bool PercussionState::isCompressorEnabled() const
-{
-        return compressor.enabled;
-}
-
-void PercussionState::setCompressorAttack(double attack)
-{
-        compressor.attack = attack;
-}
-
-void PercussionState::setCompressorRelease(double release)
-{
-        compressor.release = release;
-}
-
-void PercussionState::setCompressorThreshold(double threshold)
-{
-        compressor.threshold = threshold;
-}
-
-void PercussionState::setCompressorRatio(double ratio)
-{
-        compressor.ratio = ratio;
-}
-
-void PercussionState::setCompressorKnee(double knee)
-{
-        compressor.knee = knee;
-}
-
-void PercussionState::setCompressorMakeup(double makeup)
-{
-        compressor.makeup = makeup;
-}
-
-double PercussionState::getCompressorAttack() const
-{
-        return compressor.attack;
-}
-
-double PercussionState::getCompressorRelease() const
-{
-        return compressor.release;
-}
-
-double PercussionState::getCompressorThreshold() const
-{
-        return compressor.threshold;
-}
-
-double PercussionState::getCompressorRatio() const
-{
-        return compressor.ratio;
-}
-
-double PercussionState::getCompressorKnee() const
-{
-        return compressor.knee;
-}
-
-double PercussionState::getCompressorMakeup() const
-{
-        return compressor.makeup;
-}
-
 void PercussionState::enableDistortion(bool enable)
 {
         distortion.enabled = enable;
@@ -1018,7 +975,7 @@ void PercussionState::oscJson(std::ostringstream &jsonStream) const
                 if (val.second.function == GeonkickApi::FunctionType::Sample && !val.second.sample.empty())
                         jsonStream <<  "\"sample\": \"" << toBase64F(val.second.sample) << "\"," << std::endl;
                 jsonStream <<  "\"function\": " << static_cast<int>(val.second.function) << "," << std::endl;
-                jsonStream <<  "\"phase\": " 
+                jsonStream <<  "\"phase\": "
                            << val.second.phase << ", " << std::endl;
                 jsonStream <<  "\"seed\": " << val.second.seed << ", " << std::endl;
                 envelopeToJson(jsonStream,
@@ -1037,13 +994,18 @@ void PercussionState::oscJson(std::ostringstream &jsonStream) const
                                val.second.pitchShift,
                                val.second.pitchShiftEnvelope);
                 jsonStream << "," << std::endl;
+                envelopeToJson(jsonStream,
+                               "noise_density_env",
+                               val.second.noiseDensity,
+                               val.second.noiseDensityEnvelope);
+                jsonStream << "," << std::endl;
                 jsonStream << "\"filter\": {" << std::endl;
                 jsonStream << "\"enabled\": " << (val.second.isFilterEnabled ? "true" : "false");
                 jsonStream << ", " << std::endl;
                 jsonStream << "\"type\": " << static_cast<int>(val.second.filterType) << ", " << std::endl;
-                jsonStream << "\"cutoff\": " 
+                jsonStream << "\"cutoff\": "
                            << val.second.filterFrequency << ", " << std::endl;
-		jsonStream << "\"apply_type\": " 
+		jsonStream << "\"apply_type\": "
                            << static_cast<int>(val.second.cutOffEnvelopeApplyType) << ", " << std::endl;
                 jsonStream << "\"cutoff_env\": [";
                 bool first = true;
@@ -1056,7 +1018,7 @@ void PercussionState::oscJson(std::ostringstream &jsonStream) const
                                    << " , "  << point.y() << "]";
                 }
                 jsonStream << "], " << std::endl;
-                jsonStream << "\"factor\": " 
+                jsonStream << "\"factor\": "
                            << val.second.filterFactor << "," << std::endl;
 		jsonStream << "\"qfactor_env\": [";
                 first = true;
@@ -1082,6 +1044,7 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
 	jsonStream << "\"id\": " << getId() << "," << std::endl;
         jsonStream << "\"channel\": " << getChannel() << "," << std::endl;
         jsonStream << "\"midiChannel\": " << static_cast<int>(getMidiChannel()) << "," << std::endl;
+        jsonStream << "\"noteOffEnabled\": " << (isNoteOffEnabled() ? "true" : "false") << "," << std::endl;
         jsonStream << "\"mute\": " << (isMuted() ? "true" : "false") << "," << std::endl;
         jsonStream << "\"solo\": " << (isSolo() ? "true" : "false") << "," << std::endl;
         jsonStream << "\"name\": \"" << getName() << "\"," << std::endl;
@@ -1109,7 +1072,7 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                         jsonStream  << layersAmplitude[i];
         }
         jsonStream << "]," << std::endl;
-        jsonStream << "\"limiter\": " 
+        jsonStream << "\"limiter\": "
                    << getLimiterValue() << ", " << std::endl;
         jsonStream << "\"tuned_output\": " << (isOutputTuned() ? "true" : "false") << ", " << std::endl;
         jsonStream << "\"ampl_env\": {" << std::endl;
@@ -1164,29 +1127,13 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
         }
         jsonStream << "]" << std::endl; // points
         jsonStream << "}, " << std::endl;  // filter;
-        jsonStream << "\"compressor\": {" << std::endl;
-        jsonStream << "\"enabled\": " << (isCompressorEnabled() ? "true" : "false") << ", " << std::endl;
-        jsonStream << "\"attack\": " 
-                   << getCompressorAttack() << ", " << std::endl;
-        jsonStream << "\"release\": " 
-                   << getCompressorRelease() << ", " << std::endl;
-        jsonStream << "\"threshold\": " 
-                   << getCompressorThreshold() << ", " << std::endl;
-        jsonStream << "\"ratio\": " 
-                   << getCompressorRatio() << ", " << std::endl;
-        jsonStream << "\"knee\": " 
-                   << getCompressorKnee() << ", " << std::endl;
-        jsonStream << "\"makeup\": " 
-                   << getCompressorMakeup() << std::endl;
-        jsonStream << "}, " << std::endl;
-
         jsonStream << "\"distortion\": {" << std::endl;
         jsonStream << "\"enabled\": " << (isDistortionEnabled() ? "true" : "false") << ", " << std::endl;
-        jsonStream << "\"in_limiter\": " 
+        jsonStream << "\"in_limiter\": "
                    << getDistortionInLimiter()  << ", " << std::endl;
-        jsonStream << "\"volume\": " 
+        jsonStream << "\"volume\": "
                    << getDistortionVolume()  << ", " << std::endl;
-        jsonStream << "\"drive\": " 
+        jsonStream << "\"drive\": "
                    << getDistortionDrive() << ", " << std::endl;
 	jsonStream << "\"drive_env\": [" << std::endl;
 	points = getKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive);
