@@ -2,7 +2,7 @@
  * File name: percussion_view.cpp
  * Project: Geonkick (A percussion synthesizer)
  *
- * Copyright (C) 2020 Iurie Nistor 
+ * Copyright (C) 2020 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -54,6 +54,9 @@ RK_DECLARE_IMAGE_RC(copy_per_button_on);
 RK_DECLARE_IMAGE_RC(kit_midi_on);
 RK_DECLARE_IMAGE_RC(kit_midi_off);
 RK_DECLARE_IMAGE_RC(kit_midi_hover);
+RK_DECLARE_IMAGE_RC(note_off_unpressed);
+RK_DECLARE_IMAGE_RC(note_off_hover);
+RK_DECLARE_IMAGE_RC(note_off_pressed);
 
 PercussionLimiter::PercussionLimiter(GeonkickWidget *parent)
         : GeonkickSlider(parent)
@@ -109,6 +112,7 @@ KitPercussionView::KitPercussionView(KitWidget *parent,
         , playButton{nullptr}
         , muteButton{nullptr}
         , soloButton{nullptr}
+        , noteOffButton{nullptr}
         , percussionLimiter{nullptr}
 {
         setSize(parent->width(), 21);
@@ -132,7 +136,7 @@ void KitPercussionView::createView()
 
         // Midi channel spinbox.
         midiChannelSpinBox = new RkSpinBox(this);
-                midiChannelSpinBox->setTextColor({250, 250, 250});
+        midiChannelSpinBox->setTextColor({250, 250, 250});
         midiChannelSpinBox->setBackgroundColor({60, 57, 57});
         midiChannelSpinBox->upControl()->setBackgroundColor({50, 47, 47});
         midiChannelSpinBox->upControl()->setTextColor({100, 100, 100});
@@ -166,20 +170,36 @@ void KitPercussionView::createView()
                             RkButton::State::UnpressedHover);
         RK_ACT_BIND(keyButton, toggled, RK_ACT_ARGS(bool pressed), this, showMidiPopup());
         percussionContainer->addWidget(keyButton);
-        percussionContainer->addSpace(10);
+        percussionContainer->addSpace(5);
+
+        // Note off button
+        noteOffButton = new RkButton(this);
+        noteOffButton->setType(RkButton::ButtonType::ButtonCheckable);
+        noteOffButton->setSize(23, 16);
+        noteOffButton->setImage(RkImage(noteOffButton->size(), RK_IMAGE_RC(note_off_unpressed)),
+                                RkButton::State::Unpressed);
+        noteOffButton->setImage(RkImage(noteOffButton->size(), RK_IMAGE_RC(note_off_hover)),
+                                RkButton::State::UnpressedHover);
+        noteOffButton->setImage(RkImage(noteOffButton->size(), RK_IMAGE_RC(note_off_pressed)),
+                                RkButton::State::Pressed);
+        noteOffButton->setImage(RkImage(noteOffButton->size(), RK_IMAGE_RC(note_off_hover)),
+                                RkButton::State::PressedHover);
+        noteOffButton->show();
+        percussionContainer->addWidget(noteOffButton);
+        percussionContainer->addSpace(5);
 
         // Remove button
         removeButton = new RkButton(this);
         removeButton->setType(RkButton::ButtonType::ButtonPush);
         removeButton->setSize(16, 16);
         removeButton->setImage(RkImage(removeButton->size(), RK_IMAGE_RC(remove_per_button)),
-                             RkButton::State::Unpressed);
+                               RkButton::State::Unpressed);
         removeButton->setImage(RkImage(removeButton->size(), RK_IMAGE_RC(remove_per_button_hover)),
-                             RkButton::State::UnpressedHover);
+                               RkButton::State::UnpressedHover);
         removeButton->setImage(RkImage(removeButton->size(), RK_IMAGE_RC(remove_per_button_on)),
-                             RkButton::State::Pressed);
+                               RkButton::State::Pressed);
         removeButton->setImage(RkImage(removeButton->size(), RK_IMAGE_RC(remove_per_button_hover)),
-                             RkButton::State::PressedHover);
+                               RkButton::State::PressedHover);
         removeButton->show();
         percussionContainer->addWidget(removeButton);
         percussionContainer->addSpace(3);
@@ -198,7 +218,7 @@ void KitPercussionView::createView()
                              RkButton::State::PressedHover);
         copyButton->show();
         percussionContainer->addWidget(copyButton);
-        percussionContainer->addSpace(10);
+        percussionContainer->addSpace(5);
 
         // Limiter
         percussionLimiter = new PercussionLimiter(this);
@@ -260,10 +280,12 @@ void KitPercussionView::createView()
 
 void KitPercussionView::updateView()
 {
-        percussionLimiter->onSetValue(percussionModel->limiter());
+        percussionLimiter->onSetValue(percussionModel->limiter(), 55.0 * 100.0 / 75);
         muteButton->setPressed(percussionModel->isMuted());
         soloButton->setPressed(percussionModel->isSolo());
+        noteOffButton->setPressed(percussionModel->isNoteOffEnabled());
         size_t nMidiChannels = percussionModel->numberOfMidiChannels();
+        midiChannelSpinBox->clear();
         midiChannelSpinBox->addItem("Any");
         for (size_t i = 0; i < nMidiChannels; i++)
                 midiChannelSpinBox->addItem(std::to_string(i + 1));
@@ -282,18 +304,20 @@ void KitPercussionView::setModel(PercussionModel *model)
         RK_ACT_BIND(removeButton, released, RK_ACT_ARGS(), this, remove());
         RK_ACT_BIND(copyButton, released, RK_ACT_ARGS(), percussionModel, copy());
         RK_ACT_BIND(playButton, pressed, RK_ACT_ARGS(), percussionModel, play());
+        RK_ACT_BIND(noteOffButton, toggled, RK_ACT_ARGS(bool toggled), percussionModel, enableNoteOff(toggled));
         RK_ACT_BIND(muteButton, toggled, RK_ACT_ARGS(bool toggled), percussionModel, mute(toggled));
         RK_ACT_BIND(soloButton, toggled, RK_ACT_ARGS(bool toggled), percussionModel, solo(toggled));
         RK_ACT_BIND(percussionLimiter, valueUpdated, RK_ACT_ARGS(int val), percussionModel, setLimiter(val));
         RK_ACT_BIND(percussionModel, nameUpdated, RK_ACT_ARGS(std::string name), this, update());
         RK_ACT_BIND(percussionModel, keyUpdated, RK_ACT_ARGS(KeyIndex index), this, updateView());
         RK_ACT_BIND(percussionModel, channelUpdated, RK_ACT_ARGS(int val), this, update());
-        RK_ACT_BIND(percussionModel, limiterUpdated, RK_ACT_ARGS(int val), percussionLimiter, onSetValue(val));
+        RK_ACT_BIND(percussionModel, limiterUpdated, RK_ACT_ARGS(int val), percussionLimiter, onSetValue(val, 55.0 * 100.0 / 75));
         RK_ACT_BIND(percussionModel, muteUpdated, RK_ACT_ARGS(bool b), muteButton, setPressed(b));
         RK_ACT_BIND(percussionModel, soloUpdated, RK_ACT_ARGS(bool b), soloButton, setPressed(b));
         RK_ACT_BIND(percussionModel, selected, RK_ACT_ARGS(), this, update());
         RK_ACT_BIND(percussionModel, modelUpdated, RK_ACT_ARGS(), this, updateView());
         RK_ACT_BIND(percussionModel, midiChannelUpdated, RK_ACT_ARGS(int val), this, update());
+        RK_ACT_BIND(percussionModel, noteOffUpdated, RK_ACT_ARGS(bool b), this, update());
         updateView();
 }
 
