@@ -28,7 +28,7 @@
 
 #include "pluginterfaces/base/funknown.h"
 
-#include <RkLog.h>
+//#include <RkLog.h>
 
 using namespace Steinberg;
 static const FUID GKickVstProcessorUID(0xE2CC9532, 0x90E863F0, 0x6405E139, 0x24E34B24);
