@@ -36,6 +36,9 @@ PresetBrowserModel::PresetBrowserModel(RkObject *parent, GeonkickApi *api)
         , presetPageIndex{0}
         , numberOfPresetColumns{3}
         , rowsPerColumn{12}
+        , folderSelectedRaw{0}
+        , presetSelectedRaw{0}
+        , presetSelectedColumn{0}
         , selectedFolder{geonkickApi->getPresetFolder(0)}
         , selectedPreset{nullptr}
 {
@@ -68,9 +71,11 @@ PresetFolder* PresetBrowserModel::getPresetFolder(int row) const
 
 Preset* PresetBrowserModel::getPreset(int row, int column) const
 {
-        if (column > 0 && selectedFolder) {
-                return selectedFolder->preset(presetPage() * numberOfPresetColumns * rowsPerColumn
-                                              + (column - 1) * rowsPerColumn + row);
+        if (column > 0 && row >=0 && row < static_cast<int>(rowsPerColumn)
+            && selectedFolder) {
+                const auto index = presetPage() * numberOfPresetColumns * rowsPerColumn
+                        + (column - 1) * rowsPerColumn + row;
+                return selectedFolder->preset(index);
         } else {
                 return nullptr;
         }
@@ -161,12 +166,19 @@ void PresetBrowserModel::select(size_t row, size_t column)
                         selectedFolder = getPresetFolder(row);
                         selectedPreset = nullptr;
                         presetPageIndex = 0;
-                        if (selectedFolder)
+                        if (selectedFolder) {
+                                folderSelectedRaw    = row;
+                                presetSelectedRaw    = 0;
+                                presetSelectedColumn = 0;
                                 action folderSelected(selectedFolder);
+                        }
                 } if (column > 0 && selectedFolder) {
                         selectedPreset = getPreset(row, column);
-                        if (selectedPreset && setPreset(selectedPreset))
+                        if (selectedPreset && setPreset(selectedPreset)) {
+                                presetSelectedRaw     = row;
+                                presetSelectedColumn  = column;
                                 action presetSelected(selectedPreset);
+                        }
                 }
         }
 }
@@ -234,6 +246,11 @@ PresetFolder* PresetBrowserModel::currentSelectedFolder() const
         return selectedFolder;
 }
 
+Preset* PresetBrowserModel::currentSelectedPreset() const
+{
+        return selectedPreset;
+}
+
 bool PresetBrowserModel::removeSelectedFolder()
 {
         if (selectedFolder && geonkickApi->removePresetFolder(selectedFolder)) {
@@ -250,3 +267,49 @@ GeonkickApi* PresetBrowserModel::getGeonkickApi() const
 {
         return geonkickApi;
 }
+
+void PresetBrowserModel::selectPreviousFolder()
+{
+        if (getPresetFolder(folderSelectedRaw - 1)) {
+                select(folderSelectedRaw - 1, 0);
+        } else {
+                folderPreviousPage();
+                select(0, 0);
+        }
+        select(0, 1);
+}
+
+void PresetBrowserModel::selectNextFolder()
+{
+        if (getPresetFolder(folderSelectedRaw + 1)) {
+                select(folderSelectedRaw + 1, 0);
+        } else {
+                folderNextPage();
+                select(0, 0);
+        }
+        select(0, 1);
+}
+
+void PresetBrowserModel::selectPreviousPreset()
+{
+        if (getPreset(presetSelectedRaw - 1, presetSelectedColumn))
+                select(presetSelectedRaw - 1, presetSelectedColumn);
+        else if (getPreset(rows() - 1, presetSelectedColumn - 1))
+                select(rows() - 1, presetSelectedColumn - 1);
+        else {
+                previousPresetPage();
+                select(rows() - 1, columns() - 1);
+        }
+}
+
+void PresetBrowserModel::selectNextPreset()
+{
+        if (getPreset(presetSelectedRaw + 1, presetSelectedColumn)) {
+                select(presetSelectedRaw + 1, presetSelectedColumn);
+        } else if (getPreset(0, presetSelectedColumn + 1)) {
+                select(0, presetSelectedColumn + 1);
+        } else {
+                nextPresetPage();
+                select(0, 1);
+        }
+}
