@@ -40,6 +40,7 @@ class PresetBrowserView: public GeonkickWidget {
         void mouseMoveEvent(RkMouseEvent *event) override;
         void paintWidget(RkPaintEvent *event) override;
         void updatePageButtons();
+        void addCustomFolder();
 
  private:
         PresetBrowserModel *browserModel;
@@ -53,6 +54,8 @@ class PresetBrowserView: public GeonkickWidget {
         RkButton* prevPresetPageButton;
         RkButton* nextFolderPageButton;
         RkButton* prevFolderPageButton;
+        RkButton* addFolderButton;
+        RkButton* removeFolderButton;
         RkContainer *bottomContainer;
 };
 
