@@ -7,9 +7,10 @@ int main(int arc, char **argv)
     RK_LOG_INFO("called");
     RkMain app(arc, argv);
 
-    auto widget = new RkWidget(&app);
+    auto widget = new RkWidget(app);
     widget->setTitle("Hello!");
-	widget->setBackgroundColor({80, 80, 80});
+    widget->setBackgroundColor({80, 122, 10});
+    widget->setSize(400, 500);
     widget->show();
 
     int res = app.exec();
