@@ -26,14 +26,15 @@
 
 #include "Rk.h"
 
-struct RK_EXPORT RkCanvasInfo;
+struct RkCanvasInfo;
 
-class RK_EXPORT RkCanvas {
+class RkCanvas {
  public:
         RkCanvas() = default;
         virtual ~RkCanvas() = default;
-        virtual const RkCanvasInfo* getCanvasInfo() const = 0;
-        virtual void freeCanvasInfo() = 0;
+        virtual RkCanvasInfo* getCanvasInfo() const = 0;
+        virtual void freeCanvasInfo();
+        virtual bool isSystemWindowCanvas() const;
 };
 
 #endif // RK_CANVAS_H
