@@ -2,7 +2,7 @@
  * File name: RkMain.h
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -28,19 +28,22 @@
 
 class RkWidget;
 class RkEventQueue;
+class RkNativeWindowInfo;
 
-class RK_EXPORT RkMain {
+class RkMain {
   public:
           RkMain();
           RkMain(int argc, char **argv);
           ~RkMain();
-          bool setTopLevelWidget(RkWidget* widget);
           RkWidget* topLevelWidget(void) const;
           int exec(bool block = true);
           RkEventQueue* eventQueue() const;
+          const RkNativeWindowInfo* nativeWindowInfo() const;
+
+ protected:
+          RK_DECLARE_O_PTR(RkMain);
 
  private:
-       	  RK_DECLARE_IMPL(RkMain);
           RK_DISABLE_COPY(RkMain);
           RK_DISABLE_MOVE(RkMain);
 };
