@@ -30,12 +30,12 @@
 #ifdef RK_OS_WIN
 #include <windows.h>
 
-struct RK_EXPORT RkWindowId {
+struct RkWindowId {
         RkWindowId(HWND arg = nullptr) : id{arg} {}
         HWND id;
 };
 
-class RK_EXPORT RkNativeWindowInfo {
+class RkNativeWindowInfo {
  public:
  RkNativeWindowInfo(HWND arg = nullptr)
          : window(arg)
@@ -49,10 +49,10 @@ class RK_EXPORT RkNativeWindowInfo {
         std::string className;
 };
 
-HINSTANCE RK_EXPORT rk_win_api_instance();
-LPCSTR RK_EXPORT rk_win_api_class_name();
-RkNativeWindowInfo RK_EXPORT rk_from_native_win(HWND window, HINSTANCE instance, LPCSTR className);
-RkWindowId RK_EXPORT rk_id_from_win(HWND window);
+HINSTANCE rk_win_api_instance();
+LPCSTR rk_win_api_class_name();
+RkNativeWindowInfo rk_from_native_win(HWND window, HINSTANCE instance, LPCSTR className);
+RkWindowId rk_id_from_win(HWND window);
 
 #define RK_WIN_MESSAGE_PAINT (WM_USER + 0x0001)
 
@@ -60,13 +60,13 @@ RkWindowId RK_EXPORT rk_id_from_win(HWND window);
 #else // X11 as default
 #include <X11/Xlib.h>
 
-struct RK_EXPORT RkWindowId {
+struct RkWindowId {
         Window id;
 };
 
 struct _DndClass;
 
-class RK_EXPORT RkNativeWindowInfo
+class RkNativeWindowInfo
 {
  public:
 
@@ -82,10 +82,10 @@ class RK_EXPORT RkNativeWindowInfo
         double scaleFactor;
 };
 
-RkNativeWindowInfo RK_EXPORT rk_from_native_x11(Display* display,
-                                                int screenNumber,
-                                                Window window);
-RkWindowId RK_EXPORT rk_id_from_x11(Window window);
+RkNativeWindowInfo rk_from_native_x11(Display* display,
+                                      int screenNumber,
+                                      Window window);
+RkWindowId rk_id_from_x11(Window window);
 
 #endif // X11
 #endif // RK_PLATFORM_H
