@@ -29,7 +29,7 @@
 
 class RkEventQueue;
 
-class RK_EXPORT RkTimer: public RkObject {
+class RkTimer: public RkObject {
   public:
         explicit RkTimer(RkObject *parent, long unsigned int interval = 0);
         virtual ~RkTimer();
@@ -46,11 +46,11 @@ class RK_EXPORT RkTimer: public RkObject {
         long long unsigned int getCurrentTime() const;
 
  private:
-        RK_DISABLE_COPY(RkTimer);
-        RK_DISABLE_MOVE(RkTimer);
         long long unsigned int timerInterval;
         bool timerStarted;
         long long unsigned int lastTime;
+        RK_DISABLE_COPY(RkTimer);
+        RK_DISABLE_MOVE(RkTimer);
 };
 
 #endif // RK_TIMER_H
