@@ -2,7 +2,7 @@
  * File name: RkEventQueueImpl.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -21,7 +21,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include "RkWidget.h"
+#include "RkSystemWindow.h"
 #include "RkWidgetImpl.h"
 #include "RkEventQueueImpl.h"
 #include "RkTimer.h"
@@ -38,6 +38,7 @@
 
 RkEventQueue::RkEventQueueImpl::RkEventQueueImpl(RkEventQueue* queueInterface)
         : inf_ptr{queueInterface}
+        , systemWindow{nullptr}
 #ifdef RK_OS_WIN
         , platformEventQueue{std::make_unique<RkEventQueueWin>()}
 #elif RK_OS_MAC
@@ -54,6 +55,29 @@ RkEventQueue::RkEventQueueImpl::~RkEventQueueImpl()
         RK_LOG_DEBUG("called");
 }
 
+RkSystemWindow* RkEventQueue::RkEventQueueImpl::setTopWidget(RkWidget *widget,
+                                                  const RkNativeWindowInfo* parent)
+{
+        if (!systemWindow) {
+                systemWindow = std::make_unique<RkSystemWindow>(widget, parent);
+#ifdef RK_OS_WIN
+		systemWindow->setEventQueue(inf_ptr);
+#else
+                platformEventQueue->setDisplay(systemWindow->nativeWindowInfo()->display);
+#endif
+                addObject(widget);
+                RK_IMPL_PTR(widget)->setSystemWindow(systemWindow.get());
+        }
+        return systemWindow.get();
+}
+
+RkSystemWindow* RkEventQueue::RkEventQueueImpl::getSystemWindow() const
+{
+        if (systemWindow)
+                return systemWindow.get();
+        return nullptr;
+}
+
 bool RkEventQueue::RkEventQueueImpl::objectExists(RkObject *obj) const
 {
         return objectsList.find(obj) != objectsList.end();
@@ -61,43 +85,12 @@ bool RkEventQueue::RkEventQueueImpl::objectExists(RkObject *obj) const
 
 void RkEventQueue::RkEventQueueImpl::addObject(RkObject *obj)
 {
-        RK_LOG_DEBUG("add object: " << obj);
  	if (!obj || objectExists(obj))
  		return;
 
-        if (obj->type() == Rk::ObjectType::Widget) {
-                RK_LOG_DEBUG("obj " << obj << " is widget");
-                auto widgetImpl = dynamic_cast<RkWidget::RkWidgetImpl*>(obj->o_ptr.get());
-                if (!widgetImpl) {
-                        RK_LOG_ERROR("can't cast o_ptr to RkWidgetImpl");
-                        return;
-                }
-				
- #ifdef RK_OS_WIN
- #elif RK_OS_MAC
- #else
-                // Set the display from the top window.
-                if (!widgetImpl->parent() && !platformEventQueue->display()) {
-                        RK_LOG_DEBUG("widget " << obj << " is top window");
-                        platformEventQueue->setDisplay(widgetImpl->nativeWindowInfo()->display);
-                }
- #endif
-
-                RK_LOG_DEBUG("add widget window id");
-                auto id = (EventQueueWindowId)(widgetImpl->nativeWindowInfo()->window);
-                windowIdsMap.insert({id, obj});
-                if (static_cast<int>(widgetImpl->windowFlags())
-                    & static_cast<int>(Rk::WindowFlags::Popup)) {
-                        popupList.insert({id, obj});
-                        RK_LOG_DEBUG("poup added: " << obj);
-                }
-        }
-
         objectsList.insert(obj);
-        if (!obj->eventQueue()) {
-                RK_LOG_DEBUG("obj->setEventQueue: " << obj->eventQueue());
+        if (!obj->eventQueue())
                 obj->setEventQueue(inf_ptr);
-        }
 }
 
 void RkEventQueue::RkEventQueueImpl::addShortcut(RkObject *obj,
@@ -137,20 +130,6 @@ void RkEventQueue::RkEventQueueImpl::removeObject(RkObject *obj)
         if (objectsList.find(obj) != objectsList.end()) {
                 objectsList.erase(obj);
                 removeObjectShortcuts(obj);
-                if (obj->type() == Rk::ObjectType::Widget) {
-                        auto widgetImpl = dynamic_cast<RkWidget::RkWidgetImpl*>(obj->o_ptr.get());
-                        if (!widgetImpl) {
-                                RK_LOG_ERROR("can't cast o_ptr to RkWidgetImpl");
-                                return;
-                        }
-                        auto id = (EventQueueWindowId)(widgetImpl->nativeWindowInfo()->window);
-                        if (windowIdsMap.find(id) != windowIdsMap.end()) {
-                                RK_LOG_DEBUG("widget id removed from queue");
-                                windowIdsMap.erase(id);
-                                if (popupList.find(id) != popupList.end())
-                                        popupList.erase(id);
-                        }
-                }
         }
 }
 
@@ -162,112 +141,84 @@ void RkEventQueue::RkEventQueueImpl::removeObjectShortcuts(RkObject *obj)
         }
 }
 
-RkWidget* RkEventQueue::RkEventQueueImpl::findWidget(const RkWindowId &id) const
-{
-        auto it = windowIdsMap.find((EventQueueWindowId)(id.id));
-        if (it != windowIdsMap.end()) {
-                if (it->second->type() == Rk::ObjectType::Widget) {
-                        auto widget = dynamic_cast<RkWidget*>(it->second);
-                        if (!widget) {
-                                RK_LOG_ERROR("can't cast RkObject[" << it->second << "] to RkWidget");
-                                return nullptr;
-                        }
-                        return widget;
-                }
-        }
-
-        return nullptr;
-}
-
 void RkEventQueue::RkEventQueueImpl::postEvent(RkObject *obj, std::unique_ptr<RkEvent> event)
 {
-        eventsQueue.push_back({obj, std::move(event)});
-}
-
-void RkEventQueue::RkEventQueueImpl::postEvent(const RkWindowId &id, std::unique_ptr<RkEvent> event)
-{
-        auto it = windowIdsMap.find((EventQueueWindowId)(id.id));
-        if (it != windowIdsMap.end())
-                eventsQueue.push_back({it->second, std::move(event)});
-}
-
-void RkEventQueue::RkEventQueueImpl::processEvent(RkObject *obj, RkEvent *event)
-{
-        // Do not process events for objects that were removed from the event queue.
-        if (objectExists(obj))
-                obj->event(event);
+        if (obj && event && objectExists(obj)) {
+                std::lock_guard<std::mutex> lock(eventsQueueMutex);
+                eventsQueue.push_back({obj, std::move(event)});
+        }
 }
 
 void RkEventQueue::RkEventQueueImpl::processEvents()
 {
-#ifdef RK_OS_WIN
-#elif RK_OS_MAC
-#else
-        auto events = platformEventQueue->getEvents();
-        if (!events.empty()) {
-                for (auto &event: events) {
-                        auto widget = findWidget(event.first);
-                        if (widget) {
-                                auto pair = std::make_pair<RkObject*,
-                                            std::unique_ptr<RkEvent>>(widget, std::move(event.second));
-                                eventsQueue.push_back(std::move(pair));
+        if (systemWindow) {
+                auto systemEvents = platformEventQueue->getEvents();
+                for (auto &event: systemEvents) {
+                        auto widgetEvents = systemWindow->processEvent(event.get());
+                        for (auto &e: widgetEvents) {
+                                if (e.first && e.first->isVisible() && e.second)
+                                        postEvent(e.first, std::move(e.second));
                         }
                 }
         }
-        events.clear();
-#endif
 
         /**
-         * Moving events in a separeted queue for processing
+         * Move events in a separeted queue for processing
          * because during the processing the execution of some events
          * may add new events into the queue and this for
-         * in some cases can lead to a infinite looping.
+         * in some cases can lead to a infinite loop.
          */
-        auto queue = std::move(eventsQueue);
+        decltype(eventsQueue) queue;
+        {
+                std::lock_guard<std::mutex> lock(eventsQueueMutex);
+                queue = std::move(eventsQueue);
+        }
+
+        bool repaintSystemWindow = false;
         for (const auto &e: queue) {
+                if (!objectExists(e.first))
+                        continue;
+
                 if (e.second->type() == RkEvent::Type::KeyPressed
                     || e.second->type() == RkEvent::Type::KeyReleased) {
                         processShortcuts(dynamic_cast<RkKeyEvent*>(e.second.get()));
                 }
-                if (!popupList.empty() && dynamic_cast<RkWidget*>(e.first))
+
+                if (!popupList.empty() && dynamic_cast<RkWidget*>(e.first)
+                    && e.second->type() == RkEvent::Type::MouseButtonPress)
                         processPopups(dynamic_cast<RkWidget*>(e.first), e.second.get());
-                processEvent(e.first, e.second.get());
+
+                if (e.first) {
+                        RK_IMPL_PTR(e.first)->event(e.second.get());
+                        if (e.second->type() == RkEvent::Type::Paint)
+                                repaintSystemWindow = true;
+                }
+        }
+
+        if (systemWindow && repaintSystemWindow) {
+                for (auto it = popupList.begin(); it != popupList.end(); ++it) {
+                        auto event = std::make_unique<RkPaintEvent>();
+                        RK_IMPL_PTR((*it))->event(event.get());
+                }
+                systemWindow->update();
         }
 }
 
 void RkEventQueue::RkEventQueueImpl::processPopups(RkWidget *widget, RkEvent* event)
 {
-        if (event->type() == RkEvent::Type::MouseButtonPress) {
-                for (auto it = popupList.begin(); it != popupList.end();) {
-                        auto w = static_cast<RkWidget*>((*it).second);
-                        if (widget != w && !w->isChild(widget)) {
-                                RK_LOG_DEBUG("w->close()");
-                                w->close();
-                                it = popupList.erase(it);
-                        } else {
-                                ++it;
-                        }
+        for (auto it = popupList.begin(); it != popupList.end();) {
+                auto w = static_cast<RkWidget*>(*it);
+                if (widget != w && !w->isAncestorOf(widget)) {
+                        w->close();
+                        it = popupList.erase(it);
+                } else {
+                        ++it;
                 }
         }
 }
 
-bool RkEventQueue::RkEventQueueImpl::isTopWidget(RkObject *obj) const
-{
-        if (objectExists(obj)) {
-                auto widget = dynamic_cast<RkWidget*>(obj);
-                if (widget && widget->getTopWidget() == widget)
-                        return true;
-        }
-        return false;
-}
-
 void RkEventQueue::RkEventQueueImpl::processShortcuts(RkKeyEvent *event)
 {
-        if (!event) {
-                RK_LOG_ERROR("wrong arguments");
-                return;
-        }
-
         if (static_cast<RkKeyEvent*>(event)->isShortcut())
                 return;
 
@@ -283,8 +234,7 @@ void RkEventQueue::RkEventQueueImpl::processShortcuts(RkKeyEvent *event)
                         shurtcutEvent->setKey(event->key());
                         shurtcutEvent->setModifiers(event->modifiers());
                         shurtcutEvent->setShortcut();
-                        auto pair = std::make_pair(obj, std::move(shurtcutEvent));
-                        eventsQueue.push_back(std::move(pair));
+                        RK_IMPL_PTR(obj)->event(shurtcutEvent.get());
                 }
         } else {
                 RK_LOG_DEBUG("can't find shortcut");
@@ -297,28 +247,6 @@ void RkEventQueue::RkEventQueueImpl::postAction(std::unique_ptr<RkAction> act)
         actionsQueue.push_back(std::move(act));
 }
 
-void RkEventQueue::RkEventQueueImpl::processActions()
-{
-        decltype(actionsQueue) q;
-        {
-                std::lock_guard<std::mutex> lock(actionsQueueMutex);
-                /**
-                 * Moving actions in a separeted queue for processing
-                 * because during the processing the execution of some actions
-                 * may add new actions into the queue and this for
-                 * in some cases can lead to a infinite looping.
-                 */
-                q = std::move(actionsQueue);
-        }
-
-        for (const auto &act: q) {
-                // Do not process actions for objects that were removed from the event queue.
-                if (!act->object() || objectExists(act->object())) {
-                        act->call();
-                }
-        }
-}
-
 void RkEventQueue::RkEventQueueImpl::subscribeTimer(RkTimer *timer)
 {
         timersList.insert(timer);
@@ -338,52 +266,49 @@ void RkEventQueue::RkEventQueueImpl::processTimers()
         }
 }
 
-void RkEventQueue::RkEventQueueImpl::clearEvents(const RkObject *obj)
+void RkEventQueue::RkEventQueueImpl::processActions()
 {
-        if (!obj)
-                return;
-        RK_LOG_DEBUG("clear object " << obj << " events");
-        eventsQueue.erase(std::remove_if(eventsQueue.begin(),
-                                         eventsQueue.end(),
-                                         [obj](std::pair<RkObject*, std::unique_ptr<RkEvent>> &ev) {
-                                                 if (ev.first == obj) {
-                                                         RK_LOG_DEBUG("clear: [obj: " << obj << "] ev: "
-                                                                      << ev.second.get());
-                                                         return true;
-                                                 }
-                                                 return false;
-                                         })
-                          , eventsQueue.end());
-}
-
-void RkEventQueue::RkEventQueueImpl::clearActions(const RkObject *obj)
-{
-        if (!obj)
-                return;
-
-        std::lock_guard<std::mutex> lock(actionsQueueMutex);
-        actionsQueue.erase(std::remove_if(actionsQueue.begin(), actionsQueue.end(),
-                                          [obj](const std::unique_ptr<RkAction> &act)
-                                          {
-                                                  if (act->object() && act->object() == obj) {
-                                                          RK_LOG_DEBUG("clear: [obj: " << obj << "] act: "
-                                                                       << act.get());
-                                                          return true;
-
-                                                  }
-                                                  return false;
-                                          })
-                           , actionsQueue.end());
-}
-
-RkObject* RkEventQueue::RkEventQueueImpl::findObjectByName(const std::string &name) const
-{
-        // TODO: use less complexity O(1) with hashtable.
-        for (auto it = objectsList.cbegin(); it != objectsList.cend(); ++it) {
-                if ((*it)->name() == name)
-                        return *it;
+        decltype(actionsQueue) q;
+        {
+                std::lock_guard<std::mutex> lock(actionsQueueMutex);
+                /**
+                 * Moving actions in a separeted queue for processing
+                 * because during the processing the execution of some actions
+                 * may add new actions into the queue and this for
+                 * in some cases can lead to a infinite looping.
+                 */
+                q = std::move(actionsQueue);
         }
-        return nullptr;
+
+        for (const auto &act: q) {
+                if (!act->object() || objectExists(act->object())) {
+                        act->call();
+                }
+        }
+}
+
+void RkEventQueue::RkEventQueueImpl::processQueue()
+{
+        // The order is important.
+        processTimers();
+        processActions();
+        processEvents();
+}
+
+void RkEventQueue::RkEventQueueImpl::addPopup(RkWidget* popup)
+{
+        if (std::find(popupList.begin(), popupList.end(), popup) == popupList.end())
+                popupList.push_back(popup);
+}
+
+void RkEventQueue::RkEventQueueImpl::removePopup(RkWidget* popup)
+{
+        popupList.erase(std::remove(popupList.begin(), popupList.end(), popup), popupList.end());
+}
+
+const std::vector<RkWidget*>& RkEventQueue::RkEventQueueImpl::getPopupWidgets() const
+{
+        return popupList;
 }
 
 void RkEventQueue::RkEventQueueImpl::setScaleFactor(double factor)
@@ -391,11 +316,18 @@ void RkEventQueue::RkEventQueueImpl::setScaleFactor(double factor)
         platformEventQueue->setScaleFactor(factor);
 }
 
-double RkEventQueue::RkEventQueueImpl::getScaleFactor() const
+double RkEventQueue::RkEventQueueImpl::scaleFactor() const
 {
         return platformEventQueue->getScaleFactor();
 }
 
+#ifdef RK_OS_WIN
+RkEventQueueWin* RkEventQueue::RkEventQueueImpl::getPlatformEventQueue() const
+{
+  return platformEventQueue.get();
+}
+#endif // RK_OS_WIN
+
 void RkEventQueue::RkEventQueueImpl::dispatchEvents()
 {
 #ifdef RK_OS_WIN
