@@ -2,7 +2,7 @@
  * File name: RkImage.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -29,7 +29,6 @@
 RkImage::RkImage()
         : o_ptr{std::make_unique<RkImageImpl>(this, 0, 0, nullptr)}
 {
-        RK_LOG_DEBUG("called ----------------------------------->");
 }
 
 RkImage::RkImage(int width,
@@ -38,7 +37,6 @@ RkImage::RkImage(int width,
                  Format format)
         : o_ptr{std::make_unique<RkImageImpl>(this, width, height, data, format)}
 {
-        RK_LOG_DEBUG("called ----------------------------------->");
 }
 
 RkImage::RkImage(const RkSize &size,
@@ -46,7 +44,6 @@ RkImage::RkImage(const RkSize &size,
                  Format format)
         : o_ptr{std::make_unique<RkImageImpl>(this, size.width(), size.height(), data, format)}
 {
-        RK_LOG_DEBUG("called ----------------------------------->");
 }
 
 RkImage::RkImage(std::unique_ptr<RkImageImpl> impl)
@@ -75,15 +72,6 @@ void RkImage::fill(const RkColor &color)
         o_ptr->fill(color);
 }
 
-const RkCanvasInfo* RkImage::getCanvasInfo() const
-{
-        return o_ptr->getCanvasInfo();
-}
-
-void RkImage::freeCanvasInfo()
-{
-}
-
 unsigned char* RkImage::data() const
 {
         return o_ptr->data();
@@ -118,3 +106,8 @@ bool RkImage::isNull() const
 {
         return o_ptr->isNull();
 }
+
+RkCanvasInfo* RkImage::getCanvasInfo() const
+{
+        return o_ptr->getCanvasInfo();
+}
