@@ -2,7 +2,7 @@
  * File name: RkLineEdit.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -28,14 +28,18 @@
 #include "RkPainter.h"
 
 RkLineEdit::RkLineEdit(RkWidget *parent, const std::string &text)
-        : RkWidget(parent, std::make_unique<RkLineEdit::RkLineEditImpl>(this, parent, text))
-        , impl_ptr{static_cast<RkLineEdit::RkLineEditImpl*>(o_ptr.get())}
+        : RkWidget(parent, std::make_unique<RkLineEditImpl>(this, parent, text))
+        , impl_ptr{static_cast<RkLineEditImpl*>(o_ptr.get())}
 {
         setPointerShape(Rk::PointerShape::IBeam);
-        propagateGrabKey(false);
         impl_ptr->init();
 }
 
+RkLineEdit::~RkLineEdit()
+{
+        impl_ptr->uninit();
+}
+
 void RkLineEdit::setText(const std::string &text)
 {
      impl_ptr->setText(text);
@@ -180,8 +184,8 @@ void RkLineEdit::focusEvent(RkFocusEvent* event)
 	} else if (event->type() == RkEvent::Type::FocusedOut) {
                 RK_LOG_DEBUG("RkEvent::Type::FocusedOut");
 		if (impl_ptr->hasEditFocus())
-                        action editingFinished();
-                hideCursor();
+			action editingFinished();
+		hideCursor();
 	}
 }
 
@@ -214,4 +218,3 @@ void RkLineEdit::moveCursorToEnd()
         impl_ptr->moveCursorToBack();
         update();
 }
-
