@@ -2,7 +2,7 @@
  * File name: RkLineEditImpl.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -37,7 +37,7 @@ RkLineEdit::RkLineEditImpl::RkLineEditImpl(RkLineEdit *interface,
     , selectionIndex{0}
     , isSelectionMode{false}
     , cursorTimer{nullptr}
-    , isShowCursor{hasFocus()}
+    , isShowCursor{false}
     , lastCahnges{std::chrono::system_clock::now()}
     , contentsRect{0, 0, 0, 0}
     , beginX{0}
@@ -47,7 +47,6 @@ RkLineEdit::RkLineEditImpl::RkLineEditImpl(RkLineEdit *interface,
 
 RkLineEdit::RkLineEditImpl::~RkLineEditImpl()
 {
-        cursorTimer->stop();
 }
 
 void RkLineEdit::RkLineEditImpl::init()
@@ -58,6 +57,11 @@ void RkLineEdit::RkLineEditImpl::init()
         RK_ACT_BIND(cursorTimer, timeout, RK_ACT_ARGS(), this, onCursorTimeout());
 }
 
+void RkLineEdit::RkLineEditImpl::uninit()
+{
+        cursorTimer->stop();
+}
+
 void RkLineEdit::RkLineEditImpl::setText(const std::string &text)
 {
 	selectionIndex = 0;
@@ -312,7 +316,9 @@ void RkLineEdit::RkLineEditImpl::paintEvent(RkPaintEvent *event)
                 auto pen = painter.pen();
                 pen.setColor(textColor());
                 painter.setPen(pen);
-                painter.drawText(contentsRect.left() - beginX, contentsRect.top() + (contentsRect.height() - font().size()) / 2 + font().size(), text());
+                painter.drawText(contentsRect.left() - beginX,
+                                 contentsRect.top() + (contentsRect.height()
+                                 - font().size()) / 2 + font().size(), text());
 
                 // Draw cursor.
                 if (!isCursorHidden()) {
@@ -320,7 +326,8 @@ void RkLineEdit::RkLineEditImpl::paintEvent(RkPaintEvent *event)
                         pen.setColor(color());
                         painter.setPen(pen);
                         painter.drawLine(contentsRect.left() + cursorX + 1, contentsRect.top(),
-                                         contentsRect.left() + cursorX + 1, contentsRect.top() + contentsRect.height());
+                                         contentsRect.left() + cursorX + 1,
+                                         contentsRect.top() + contentsRect.height());
                 }
         }
         RkPainter paint(inf_ptr);
@@ -338,4 +345,3 @@ bool RkLineEdit::RkLineEditImpl::hasEditFocus() const
 {
 	return cursorTimer->started();
 }
-
