@@ -2,7 +2,7 @@
  * File name: RkMain.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -25,6 +25,8 @@
 #include "RkLog.h"
 #include "RkWidget.h"
 #include "RkMainImpl.h"
+#include "RkEventQueueImpl.h"
+#include "RkSystemWindow.h"
 
 RkMain::RkMain()
         : o_ptr{std::make_unique<RkMainImpl>(this)}
@@ -40,14 +42,9 @@ RkMain::~RkMain()
 {
 }
 
-bool RkMain::setTopLevelWidget(RkWidget *widget)
-{
-        return o_ptr->setTopLevelWidget(widget);
-}
-
 RkWidget* RkMain::topLevelWidget(void) const
 {
-        return o_ptr->topLevelWidget();
+        return o_ptr->topWidget();
 }
 
 RkEventQueue* RkMain::eventQueue() const
@@ -55,6 +52,11 @@ RkEventQueue* RkMain::eventQueue() const
         return o_ptr->getEventQueue();
 }
 
+const RkNativeWindowInfo* RkMain::nativeWindowInfo() const
+{
+        return RK_IMPL_PTR(o_ptr->getEventQueue())->getSystemWindow()->nativeWindowInfo();
+}
+
 int RkMain::exec(bool block)
 {
 	return o_ptr->exec(block);
