@@ -2,7 +2,7 @@
  * File name: RkPainter.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -106,6 +106,12 @@ void RkPainter::drawPolyline(const std::vector<RkPoint> &points)
                 o_ptr->drawPolyline(points);
 }
 
+void RkPainter::drawPolyline(const std::vector<RkRealPoint> &points)
+{
+        if (points.size() > 1)
+                o_ptr->drawPolyline(points);
+}
+
 void RkPainter::fillRect(const RkRect &rect, const RkColor &color)
 {
         if (rect.area() > 0)
@@ -151,3 +157,8 @@ int RkPainter::getTextWidth(const std::string &text) const
 {
         return o_ptr->getTextWidth(text);
 }
+
+void RkPainter::scale(double x, double y)
+{
+        return o_ptr->scale(x, y);
+}
