@@ -2,7 +2,7 @@
  * File name: RkPainterImpl.cpp
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -78,6 +78,11 @@ void RkPainter::RkPainterImpl::drawPolyline(const std::vector<RkPoint> &points)
         backendGraphics->drawPolyLine(points);
 }
 
+void RkPainter::RkPainterImpl::drawPolyline(const std::vector<RkRealPoint> &points)
+{
+        backendGraphics->drawPolyLine(points);
+}
+
 void RkPainter::RkPainterImpl::fillRect(const RkRect &rect, const RkColor &color)
 {
         backendGraphics->fillRect(rect, color);
@@ -124,3 +129,8 @@ void RkPainter::RkPainterImpl::applyAlpha(int alpha)
 {
         backendGraphics->applyAlpha(alpha);
 }
+
+void RkPainter::RkPainterImpl::scale(double x, double y)
+{
+        backendGraphics->scale(x, y);
+}
