@@ -2,7 +2,7 @@
  * File name: main.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -73,7 +73,7 @@ int main(int argc, char *argv[])
                 exit(EXIT_FAILURE);
         }
 
-        auto window = new MainWindow(&app, api, preset);
+        auto window = new MainWindow(app, api, preset);
         if (!window->init()) {
                 GEONKICK_LOG_ERROR("can't init main window");
                 exit(EXIT_FAILURE);
@@ -83,6 +83,6 @@ int main(int argc, char *argv[])
 #ifdef GEONKICK_OS_GNU
         munlockall();
 #endif // GEONKICK_OS_GNU
-	
+
         return res;
 }
